# -*- coding: utf_8 -*-
"""Module holding the functions for icon analysis."""

import logging
import glob
import os
import platform
import shutil
import subprocess
from pathlib import Path

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    is_dir_exists,
)

logger = logging.getLogger(__name__)


def get_icon_from_ipa(app_dict):
    """Get app icon from IPA."""
    try:
        binary = app_dict['infoplist'].get('bin')
        md5 = app_dict['md5_hash']
        bin_dir = app_dict['bin_dir']
        msg = 'Fetching IPA icon path'
        logger.info(msg)
        append_scan_status(md5, msg)
        bin_path = os.path.join(bin_dir, binary + '.app')
        if not is_dir_exists(bin_path):
            logger.warning('Could not find app binary directory')
            return
        icons = glob.glob(bin_path + '/AppIcon*png')
        if not icons:
            logger.warning('Could not find app icon')
            return
        icon_file = icons.pop()
        outfile = Path(settings.DWD_DIR) / f'{md5}-icon.png'
        app_dict['icon_path'] = outfile.name
        tools_dir = Path(settings.BASE_DIR) / 'StaticAnalyzer' / 'tools' / 'ios'
        arch = platform.machine()
        system = platform.system()
        # Uncrush PNG. CgBI -> PNG
        # https://iphonedevwiki.net/index.php/CgBI_file_format
        if system == 'Darwin':
            args = ['xcrun', '-sdk', 'iphoneos', 'pngcrush', '-q',
                    '-revert-iphone-optimizations',
                    icon_file, outfile.as_posix()]
            try:
                out = subprocess.run(args, capture_output=True)
                if b'libpng error:' in out.stdout:
                    # PNG looks normal
                    raise ValueError('PNG is not CgBI')
            except Exception:
                shutil.copy2(icon_file, outfile.as_posix())
        else:
            # Windows/Linux
            cgbipng_bin = None
            if system == 'Windows' and arch in ('AMD64', 'x86'):
                cgbipng_bin = 'CgbiPngFix.exe'
            elif system == 'Linux' and arch == 'x86_64':
                cgbipng_bin = 'CgbiPngFix_amd64'
            elif system == 'Linux' and arch == 'aarch64':
                cgbipng_bin = 'CgbiPngFix_arm64'
            if cgbipng_bin:
                cbin = tools_dir / 'CgbiPngFix' / cgbipng_bin
                args = [cbin.as_posix(), '-i',
                        icon_file, '-o', outfile.as_posix()]
                try:
                    out = subprocess.run(args, capture_output=True)
                except Exception:
                    # Fails or PNG is not crushed
                    shutil.copy2(icon_file, outfile.as_posix())
            else:
                logger.warning('CgbiPngFix not available for %s %s', system, arch)
                shutil.copy2(icon_file, outfile.as_posix())
    except Exception as exp:
        msg = 'Error Fetching IPA icon'
        logger.exception(msg)
        append_scan_status(md5, msg, repr(exp))


def get_icon_source(app_dict):
    checksum = app_dict['md5_hash']
    src_dir = app_dict['app_dir']
    """Get app icon from iOS ZIP."""
    msg = 'Fetching icon path'
    logger.info(msg)
    append_scan_status(checksum, msg)
    try:
        appiconset = []
        for dirname, _, files in os.walk(src_dir):
            for img in files:
                full_path = os.path.join(src_dir, dirname, img)
                if '__MACOSX' in full_path:
                    continue
                if '.appiconset' in full_path and img.endswith('.png'):
                    appiconset.append(full_path)
        if not appiconset:
            return
        icon_file = appiconset[0]
        outfile = Path(settings.DWD_DIR) / f'{checksum}-icon.png'
        shutil.copy2(icon_file, outfile.as_posix())
        app_dict['icon_path'] = outfile.name
    except Exception as exp:
        msg = 'Error Fetching icon'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))



# -*- coding: utf_8 -*-
"""MobSF REST API V 1."""

from django.views.decorators.csrf import csrf_exempt

from mobsf.MobSF.views.helpers import request_method
from mobsf.MobSF.views.api.api_middleware import make_api_response
from mobsf.DynamicAnalyzer.views.ios import (
    corellium_instance,
    dynamic_analyzer,
    report,
    tests_frida,
)


FAILED = 'failed'
ERROR = 'error'
INSTANCE_ID = 'instance_id'
BUNDLE_ID = 'bundle_id'


# Dynamic Analyzer APIs
@request_method(['POST'])
@csrf_exempt
def api_ios_dynamic_analysis(request):
    """POST - iOS Dynamic Analysis Entrypoint."""
    resp = dynamic_analyzer.dynamic_analysis(request, True)
    if ERROR in resp:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_ios_dynamic_analyzer(request):
    """POST - iOS Dynamic Analyzer."""
    params = {INSTANCE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = dynamic_analyzer.dynamic_analyzer(request, True)
    if ERROR in resp:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


# Corellium Instance Operations
@request_method(['POST'])
@csrf_exempt
def api_corellium_get_supported_models(request):
    """POST - Corellium Get Supported iOS Models."""
    resp = corellium_instance.get_supported_models(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_corellium_get_supported_ios_versions(request):
    """POST - Corellium Get Supported iOS Versions."""
    if 'model' not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.get_supported_os(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_corellium_create_ios_instance(request):
    """POST - Corellium Create an iOS Instance."""
    params = {'name', 'project_id', 'flavor', 'version'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.create_vm_instance(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_corellium_start_instance(request):
    """POST - Start Corellium Instance."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.start_instance(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_corellium_stop_instance(request):
    """POST - Stop Corellium Instance."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.stop_instance(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_corellium_unpause_instance(request):
    """POST - Unpause Corellium Instance."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.unpause_instance(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_corellium_reboot_instance(request):
    """POST - Reboot Corellium Instance."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.reboot_instance(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_corellium_destroy_instance(request):
    """POST - Destroy Corellium Instance."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.destroy_instance(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_corellium_instance_list_apps(request):
    """POST - Corellium Instance List Apps."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.list_apps(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


# Dynamic Analyzer Helpers
@request_method(['POST'])
@csrf_exempt
def api_setup_environment(request):
    """POST - Setup Dynamic Analyzer Environment."""
    params = {INSTANCE_ID, 'hash'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.setup_environment(
        request,
        request.POST['hash'],
        True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_run_app(request):
    """POST - Run an App."""
    params = {INSTANCE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.run_app(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_stop_app(request):
    """POST - Run an App."""
    params = {INSTANCE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.stop_app(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_remove_app(request):
    """POST - Remove an App from Device."""
    params = {INSTANCE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.remove_app(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_take_screenshot(request):
    """POST - Take a Screenshot."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.take_screenshot(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_get_app_container_path(request):
    """POST - Get App Container path."""
    if BUNDLE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.get_container_path(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_network_capture(request):
    """POST - Enable/Disable Network Capture."""
    params = {INSTANCE_ID, 'state'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.network_capture(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_live_pcap_download(request):
    """POST - Download Network Capture file."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.live_pcap_download(request, True)
    if resp.get('Content-Disposition'):
        # PCAP file http response
        return resp
    elif resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_ssh_execute(request):
    """POST - Execute commands in VM over SSH."""
    params = {INSTANCE_ID, 'cmd'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.ssh_execute(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_download_app_data(request):
    """POST - Download Application Data from Device."""
    params = {INSTANCE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.download_data(
        request,
        request.POST[BUNDLE_ID],
        True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_instance_input(request):
    """POST - Sending Touch/Swipe/Text Events."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.touch(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_system_logs(request):
    """POST - Get system logs."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.system_logs(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_file_upload(request):
    """POST - Upload file to device."""
    if INSTANCE_ID not in request.POST:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.upload_file(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


@request_method(['POST'])
@csrf_exempt
def api_device_file_download(request):
    """POST - Download file from device."""
    params = {INSTANCE_ID, 'file'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = corellium_instance.download_file(request, True)
    if resp.get('Content-Disposition'):
        # file http response
        return resp
    elif resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


# Frida APIs
@request_method(['POST'])
@csrf_exempt
def api_ios_instrument(request):
    """POST - Frida Instrument."""
    params = {
        INSTANCE_ID,
        BUNDLE_ID,
        'hash',
        'default_hooks',
        'dump_hooks',
        'auxiliary_hooks',
        'frida_code'}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = tests_frida.ios_instrument(request, True)
    if resp.get('status') == FAILED:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)


# Report APIs
@request_method(['POST'])
@csrf_exempt
def api_ios_view_report(request):
    """POST - iOS Dynamic Analysis report."""
    params = {INSTANCE_ID, BUNDLE_ID}
    if set(request.POST) < params:
        return make_api_response(
            {'error': 'Missing Parameters'}, 422)
    resp = report.ios_view_report(
        request,
        request.POST[BUNDLE_ID],
        True)
    if 'error' in resp:
        return make_api_response(resp, 500)
    return make_api_response(resp, 200)



"""Inject Frida Gadget to an APK."""
import subprocess
from pathlib import Path
from lzma import LZMAFile
from shutil import copyfileobj
from tempfile import gettempdir
import logging

import requests

from django.conf import settings

from mobsf.MobSF.utils import (
    find_java_binary,
    is_file_exists,
    is_internet_available,
    upstream_proxy,
)


logger = logging.getLogger(__name__)


class GadgetPatcher:

    def __init__(self, app_dir, tools_dir, apk_file) -> None:
        self.app_dir = app_dir
        self.tools_dir = tools_dir
        self.apk_file = apk_file
        self.output_dir = Path(self.app_dir) / 'apktool_repack'
        if (len(settings.APKTOOL_BINARY) > 0
                and is_file_exists(settings.APKTOOL_BINARY)):
            self.apktool_path = settings.APKTOOL_BINARY
        else:
            self.apktool_path = Path(self.tools_dir) / 'apktool_2.7.0.jar'

    def decompile_apk(self):
        """Decompile APK."""
        args = [find_java_binary(),
                '-Djdk.util.zip.disableZip64ExtraFieldValidation=true',
                '-jar',
                self.apktool_path,
                '--frame-path',
                gettempdir(),
                '-f', '-r', '-s', 'd',
                self.apk_file,
                '-o',
                self.output_dir]
        manifest = self.output_dir / 'AndroidManifest.xml'
        if not is_file_exists(manifest):
            # Skip if already decompiled
            subprocess.check_output(args)

    def recompile_apk(self):
        """Recompile APK."""
        args = [find_java_binary(),
                '-Djdk.util.zip.disableZip64ExtraFieldValidation=true',
                '-jar',
                self.apktool_path,
                '-f', 'b',
                '-o',
                '.patched.apk'.join(
                    self.apk_file.rsplit('.apk', 1)),
                self.output_dir]
        subprocess.check_output(args)

    def download_frida_gadget(self, frida_arch, aarch, version):
        """Download Frida Gadget."""
        gadget_bin = self.output_dir / 'lib' / {aarch} / 'libfrida-gadget.so'
        fgadget = f'frida-gadget-{version}-android-{frida_arch}.so'
        if gadget_bin.is_file():
            return True
        if not is_internet_available():
            return None
        try:
            proxies, verify = upstream_proxy('https')
        except Exception:
            logger.exception('[ERROR] Setting upstream proxy')
            return None
        try:
            response = requests.get(f'{settings.FRIDA_SERVER}{version}',
                                    timeout=5,
                                    proxies=proxies,
                                    verify=verify)
            for item in response.json()['assets']:
                if item['name'] == f'{fgadget}.xz':
                    url = item['browser_download_url']
                    break
            if not url:
                return None
            logger.info('Downloading frida-gadget %s', fgadget)
            with requests.get(url,
                              timeout=5,
                              stream=True,
                              proxies=proxies,
                              verify=verify) as r:
                with LZMAFile(r.raw) as f:
                    with open(gadget_bin, 'wb') as flip:
                        copyfileobj(f, flip)
                return True
        except Exception:
            logger.exception('[ERROR] Fetching Frida Gadget Release')
        return None



# -*- coding: utf_8 -*-
"""Module holding the functions for code analysis."""

import hashlib
import logging
import os
import re
import subprocess
from pathlib import Path

import asn1crypto

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import (
    dsa,
    ec,
    rsa,
)

from django.utils.html import escape

from mobsf.MobSF.utils import (
    append_scan_status,
    find_java_binary,
    gen_sha256_hash,
)
from mobsf.StaticAnalyzer.tools.androguard4 import (
    util,
)

logger = logging.getLogger(__name__)
ANDROID_8_1_LEVEL = 27
HIGH = 'high'
WARNING = 'warning'
INFO = 'info'
HASH_FUNCS = {
    'md5': hashlib.md5,
    'sha1': hashlib.sha1,
    'sha256': hashlib.sha256,
    'sha512': hashlib.sha512,
}


def get_hardcoded_cert_keystore(app_dic):
    """Returns the hardcoded certificate keystore."""
    app_dic['file_analysis'] = []
    checksum = app_dic['md5']
    try:
        files = app_dic.get('files') or app_dic.get('apk_files')
        msg = 'Getting Hardcoded Certificates/Keystores'
        logger.info(msg)
        append_scan_status(checksum, msg)
        findings = []
        certz = []
        key_store = []
        if not files:
            return
        for file_name in files:
            if '.' not in file_name:
                continue
            ext = Path(file_name).suffix
            if ext in ('.cer', '.pem', '.cert', '.crt',
                       '.pub', '.key', '.pfx', '.p12', '.der'):
                certz.append(escape(file_name))
            if ext in ('.jks', '.bks'):
                key_store.append(escape(file_name))
        if certz:
            desc = 'Certificate/Key files hardcoded inside the app.'
            findings.append({'finding': desc, 'files': certz})
        if key_store:
            desc = 'Hardcoded Keystore found.'
            findings.append({'finding': desc, 'files': key_store})
        app_dic['file_analysis'] = findings
    except Exception as exp:
        msg = 'Getting Hardcoded Certificates/Keystores'
        append_scan_status(checksum, msg, repr(exp))
        logger.exception(msg)


def get_cert_details(data):
    """Get certificate details."""
    certlist = []
    x509_cert = asn1crypto.x509.Certificate.load(data)
    subject = util.get_certificate_name_string(x509_cert.subject, short=True)
    certlist.append(f'X.509 Subject: {subject}')
    certlist.append(f'Signature Algorithm: {x509_cert.signature_algo}')
    valid_from = x509_cert['tbs_certificate']['validity']['not_before'].native
    certlist.append(f'Valid From: {valid_from}')
    valid_to = x509_cert['tbs_certificate']['validity']['not_after'].native
    certlist.append(f'Valid To: {valid_to}')
    issuer = util.get_certificate_name_string(x509_cert.issuer, short=True)
    certlist.append(f'Issuer: {issuer}')
    certlist.append(f'Serial Number: {hex(x509_cert.serial_number)}')
    certlist.append(f'Hash Algorithm: {x509_cert.hash_algo}')
    for k, v in HASH_FUNCS.items():
        certlist.append(f'{k}: {v(data).hexdigest()}')
    return certlist


def get_pub_key_details(data):
    """Get public key details."""
    certlist = []

    x509_public_key = serialization.load_der_public_key(
        data,
        backend=default_backend())
    alg = 'unknown'
    fingerprint = ''
    if isinstance(x509_public_key, rsa.RSAPublicKey):
        alg = 'rsa'
        modulus = x509_public_key.public_numbers().n
        public_exponent = x509_public_key.public_numbers().e
        to_hash = f'{modulus}:{public_exponent}'
    elif isinstance(x509_public_key, dsa.DSAPublicKey):
        alg = 'dsa'
        dsa_parameters = x509_public_key.parameters()
        p = dsa_parameters.parameter_numbers().p
        q = dsa_parameters.parameter_numbers().q
        g = dsa_parameters.parameter_numbers().g
        y = x509_public_key.public_numbers().y
        to_hash = f'{p}:{q}:{g}:{y}'
    elif isinstance(x509_public_key, ec.EllipticCurvePublicKey):
        alg = 'ec'
        to_hash = f'{x509_public_key.public_numbers().curve.name}:'
        to_hash = to_hash.encode('utf-8')
        # Untested, possibly wrong key size and fingerprint
        to_hash += data[25:]
    fingerprint = gen_sha256_hash(to_hash)
    certlist.append(f'PublicKey Algorithm: {alg}')
    certlist.append(f'Bit Size: {x509_public_key.key_size}')
    certlist.append(f'Fingerprint: {fingerprint}')
    return certlist


def get_signature_versions(checksum, app_path, tools_dir, signed):
    """Get signature versions using apksigner."""
    v1, v2, v3, v4 = False, False, False, False
    try:
        if not signed:
            return v1, v2, v3, v4
        logger.info('Getting Signature Versions')
        apksigner = Path(tools_dir) / 'apksigner.jar'
        args = [find_java_binary(), '-Xmx1024M',
                '-Djava.library.path=', '-jar',
                apksigner.as_posix(),
                'verify', '--verbose', app_path]
        out = subprocess.check_output(
            args, stderr=subprocess.STDOUT)
        out = out.decode('utf-8', 'ignore')
        if re.findall(r'v1 scheme \(JAR signing\): true', out):
            v1 = True
        if re.findall(r'\(APK Signature Scheme v2\): true', out):
            v2 = True
        if re.findall(r'\(APK Signature Scheme v3\): true', out):
            v3 = True
        if re.findall(r'\(APK Signature Scheme v4\): true', out):
            v4 = True
    except Exception as exp:
        msg = 'Failed to get signature versions with apksigner'
        logger.error(msg)
        append_scan_status(checksum, msg, repr(exp))
    return v1, v2, v3, v4


def apksigtool_cert(checksum, apk_path, tools_dir):
    """Get Human readable certificate with apksigtool."""
    certlist = []
    certs = []
    pub_keys = []
    signed = False
    certs_no = 0
    min_sdk = None
    av1, av2, av3, av4 = None, None, None, None
    v1, v2, v3, v4 = None, None, None, None
    try:
        from apksigtool import (
            APKSignatureSchemeBlock,
            extract_v2_sig,
            parse_apk_signing_block,
        )
        from apksigcopier import (
            extract_meta,
        )
        meta = extract_meta(apk_path)
        sig_files = [x.filename for x, _ in meta]
        if sig_files:
            av1 = True
        else:
            av1 = False
        try:
            _, sig_block = extract_v2_sig(apk_path)
            for pair in parse_apk_signing_block(sig_block).pairs:
                b = pair.value
                if isinstance(b, APKSignatureSchemeBlock):
                    signed = True
                    for signer in b.signers:
                        av2 = b.is_v2()
                        av3 = b.is_v3()
                        if b.is_v3():
                            min_sdk = signer.min_sdk
                        certs_no = len(signer.signed_data.certificates)
                        for cert in signer.signed_data.certificates:
                            d = get_cert_details(cert.data)
                            for i in d:
                                if i not in certs:
                                    certs.append(i)
                        p = get_pub_key_details(signer.public_key.data)
                        for j in p:
                            if j not in pub_keys:
                                pub_keys.append(j)
        except Exception:
            logger.warning('Failed to get signature versions with apksigtool')

        if signed:
            certlist.append('Binary is signed')
        else:
            certlist.append('Binary is not signed')
        v1, v2, v3, v4 = get_signature_versions(
            checksum,
            apk_path,
            tools_dir,
            signed)
        if signed and not (v1 or v2 or v3 or v4):
            # apksigner.jar failed to get signature versions
            logger.info('Fetching signature versions with apksigtool')
            v1, v2, v3, v4 = av1, av2, av3, av4
        certlist.append(f'v1 signature: {v1}')
        certlist.append(f'v2 signature: {v2}')
        certlist.append(f'v3 signature: {v3}')
        certlist.append(f'v4 signature: {v4}')
        certlist.extend(certs)
        certlist.extend(pub_keys)
        certlist.append(f'Found {certs_no} unique certificates')
    except Exception as exp:
        msg = 'Failed to parse code signing certificate'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
        certlist.append('Missing certificate')
    return {
        'cert_data': '\n'.join(certlist),
        'signed': signed,
        'v1': v1,
        'v2': v2,
        'v3': v3,
        'v4': v4,
        'min_sdk': min_sdk,
    }


def get_cert_data(checksum, a, app_path, tools_dir):
    """Get Human readable certificate."""
    certlist = []
    signed = False
    if a.is_signed():
        signed = True
        certlist.append('Binary is signed')
    else:
        certlist.append('Binary is not signed')
        certlist.append('Missing certificate')
    v1, v2, v3, v4 = get_signature_versions(
        checksum,
        app_path,
        tools_dir,
        signed)
    if signed and not (v1 or v2 or v3 or v4):
        # apksigner.jar failed to get signature versions
        logger.info('Fetching signature versions with androguard')
        v1, v2, v3, v4 = a.is_signed_v1(), a.is_signed_v2(), a.is_signed_v3(), None
    certlist.append(f'v1 signature: {v1}')
    certlist.append(f'v2 signature: {v2}')
    certlist.append(f'v3 signature: {v3}')
    certlist.append(f'v4 signature: {v4}')

    certs = set(a.get_certificates_der_v3() + a.get_certificates_der_v2()
                + [a.get_certificate_der(x)
                    for x in a.get_signature_names()])
    pkeys = set(a.get_public_keys_der_v3() + a.get_public_keys_der_v2())

    for cert in certs:
        certlist.extend(get_cert_details(cert))

    for public_key in pkeys:
        certlist.extend(get_pub_key_details(public_key))

    if len(certs) > 0:
        certlist.append(f'Found {len(certs)} unique certificates')

    return {
        'cert_data': '\n'.join(certlist),
        'signed': signed,
        'v1': v1,
        'v2': v2,
        'v3': v3,
        'v4': v4,
        'min_sdk': None,
    }


def cert_info(app_dic, man_dict):
    """Return certificate information."""
    try:
        msg = 'Reading Code Signing Certificate'
        logger.info(msg)
        append_scan_status(app_dic['md5'], msg)
        a = app_dic.get('androguard_apk')
        manifestfile = None
        manidat = ''
        files = []
        summary = {HIGH: 0, WARNING: 0, INFO: 0}

        if a:
            cert_data = get_cert_data(
                app_dic['md5'],
                a, app_dic['app_path'],
                app_dic['tools_dir'])
        else:
            logger.warning('androguard certificate parsing failed,'
                           ' switching to apksigtool')
            cert_data = apksigtool_cert(
                app_dic['md5'],
                app_dic['app_path'],
                app_dic['tools_dir'])

        cert_path = os.path.join(app_dic['app_dir'], 'META-INF/')
        if os.path.exists(cert_path):
            files = [f for f in os.listdir(
                cert_path) if os.path.isfile(os.path.join(cert_path, f))]
        if 'MANIFEST.MF' in files:
            manifestfile = os.path.join(cert_path, 'MANIFEST.MF')
        if manifestfile:
            with open(manifestfile, 'r', encoding='utf-8') as manifile:
                manidat = manifile.read()
        sha256_digest = bool(re.findall(r'SHA-256-Digest', manidat))
        findings = []
        if cert_data['signed']:
            summary[INFO] += 1
            findings.append((
                INFO,
                'Application is signed with a code '
                'signing certificate',
                'Signed Application'))
        else:
            summary[HIGH] += 1
            findings.append((
                HIGH,
                'Code signing certificate not found',
                'Missing Code Signing certificate'))

        if man_dict['min_sdk']:
            api_level = int(man_dict['min_sdk'])
        elif cert_data['min_sdk']:
            api_level = int(cert_data['min_sdk'])
        else:
            # API Level unknown
            api_level = None

        if cert_data['v1'] and api_level:
            status = HIGH
            summary[HIGH] += 1
            if ((cert_data['v2'] or cert_data['v3'])
                    and api_level < ANDROID_8_1_LEVEL):
                status = WARNING
                summary[HIGH] -= 1
                summary[WARNING] += 1
            findings.append((
                status,
                'Application is signed with v1 signature scheme, '
                'making it vulnerable to Janus vulnerability on '
                'Android 5.0-8.0, if signed only with v1 signature'
                ' scheme. Applications running on Android 5.0-7.0'
                ' signed with v1, and v2/v3 '
                'scheme is also vulnerable.',
                'Application vulnerable to Janus Vulnerability'))
        if re.findall(r'CN=Android Debug', cert_data['cert_data']):
            summary[HIGH] += 1
            findings.append((
                HIGH,
                'Application signed with a debug certificate. '
                'Production application must not be shipped '
                'with a debug certificate.',
                'Application signed with debug certificate'))
        if re.findall(r'Hash Algorithm: sha1', cert_data['cert_data']):
            status = HIGH
            summary[HIGH] += 1
            desc = (
                'Application is signed with SHA1withRSA. '
                'SHA1 hash algorithm is known to have '
                'collision issues.')
            title = 'Certificate algorithm vulnerable to hash collision'
            if sha256_digest:
                status = WARNING
                summary[HIGH] -= 1
                summary[WARNING] += 1
                desc += (
                    ' The manifest file indicates SHA256withRSA'
                    ' is in use.')
                title = ('Certificate algorithm might be '
                         'vulnerable to hash collision')
            findings.append((status, desc, title))
        if re.findall(r'Hash Algorithm: md5', cert_data['cert_data']):
            status = HIGH
            summary[HIGH] += 1
            desc = (
                'Application is signed with MD5. '
                'MD5 hash algorithm is known to have '
                'collision issues.')
            title = 'Certificate algorithm vulnerable to hash collision'
            findings.append((status, desc, title))
        return {
            'certificate_info': cert_data['cert_data'],
            'certificate_findings': findings,
            'certificate_summary': summary,
        }
    except Exception as exp:
        msg = 'Reading Code Signing Certificate'
        logger.exception(msg)
        append_scan_status(app_dic['md5'], msg, repr(exp))
        return {}



#!/usr/bin/env python
# Mass Static Analysis
import argparse
import logging
import os
import urllib.error
import urllib.parse
import urllib.request

import requests

logger = logging.getLogger(__name__)


def is_server_up(url):
    try:
        urllib.request.urlopen(url, timeout=5)
        return True
    except urllib.error.URLError:
        pass
    return False


def start_scan(directory, server_url, apikey, rescan='0'):
    print('\nLooking for Android/iOS/'
          'Windows binaries or source code in : ' + directory)
    logger.info('Uploading to MobSF Server')
    uploaded = []
    mimes = {
        '.apk': 'application/octet-stream',
        '.ipa': 'application/octet-stream',
        '.appx': 'application/octet-stream',
        '.zip': 'application/zip',
    }
    for filename in os.listdir(directory):
        fpath = os.path.join(directory, filename)
        _, ext = os.path.splitext(fpath)
        if ext in mimes:
            files = {'file': (filename, open(fpath, 'rb'),
                              mimes[ext], {'Expires': '0'})}
            response = requests.post(
                server_url + '/api/v1/upload',
                files=files,
                headers={'AUTHORIZATION': apikey})
            if response.status_code == 200 and 'hash' in response.json():
                logger.info('[OK] Upload OK: %s', filename)
                uploaded.append(response.json())
            else:
                logger.error('Performing Upload: %s', filename)

    logger.info('Running Static Analysis')
    for upl in uploaded:
        logger.info('Started Static Analysis on: %s', upl['file_name'])
        if rescan == '1':
            upl['re_scan'] = 1
        response = requests.post(
            server_url + '/api/v1/scan',
            data=upl,
            headers={'AUTHORIZATION': apikey})
        if response.status_code == 200:
            logger.info('[OK] Static Analysis Complete: %s', upl['file_name'])
        else:
            logger.error('Performing Static Analysis: %s', upl['file_name'])


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-d', '--directory',
                        help='Path to the directory that contains '
                             'mobile app binary/zipped source code')
    parser.add_argument(
        '-s', '--ipport', help='IP address and Port number '
                               'of a running MobSF Server. '
                               '(ex: 127.0.0.1:8000)')
    parser.add_argument(
        '-k', '--apikey', help='MobSF API Key')
    parser.add_argument(
        '-r', '--rescan', help='Run a fresh scan. '
                               'Value can be 1 or 0 (Default: 0)')
    args = parser.parse_args()

    if args.directory and args.ipport and args.apikey:
        server = args.ipport
        directory = args.directory
        server_url = 'http://' + server
        apikey = args.apikey
        rescan = args.rescan
        if not is_server_up(server_url):
            print('MobSF API Server is not running at ' + server_url)
            print('Exiting!')
            exit(0)
        # MobSF is running, start scan
        start_scan(directory, server_url, apikey, rescan)
    else:
        parser.print_help()



# -*- coding: utf_8 -*-
"""Android APK and Source Analysis."""
import logging
import shutil
from pathlib import Path

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
import mobsf.MalwareAnalyzer.views.VirusTotal as VirusTotal
from mobsf.MalwareAnalyzer.views.android import (
    apkid,
    permissions,
)
from mobsf.MalwareAnalyzer.views.MalwareDomainCheck import MalwareDomainCheck

from django.conf import settings
from django.http import HttpResponseRedirect
from django.shortcuts import render

from mobsf.MobSF.utils import (
    append_scan_status,
    file_size,
    print_n_send_error_response,
)
from mobsf.StaticAnalyzer.models import (
    StaticAnalyzerAndroid,
    StaticAnalyzerIOS,
)
from mobsf.StaticAnalyzer.views.common.binary.lib_analysis import (
    library_analysis,
)
from mobsf.StaticAnalyzer.views.android.app import (
    aapt_parse,
    androguard_parse,
    get_apk_name,
)
from mobsf.StaticAnalyzer.views.android.cert_analysis import (
    cert_info,
    get_hardcoded_cert_keystore,
)
from mobsf.StaticAnalyzer.views.android.code_analysis import code_analysis
from mobsf.StaticAnalyzer.views.android.converter import (
    apk_2_java,
    dex_2_smali,
)
from mobsf.StaticAnalyzer.views.android.db_interaction import (
    get_context_from_db_entry,
    save_get_ctx,
)
from mobsf.StaticAnalyzer.views.android.icon_analysis import (
    get_icon_apk,
    get_icon_from_src,
)
from mobsf.StaticAnalyzer.views.android.manifest_analysis import (
    manifest_analysis,
)
from mobsf.StaticAnalyzer.views.android.manifest_utils import (
    extract_manifest_data,
    get_parsed_manifest,
)
from mobsf.StaticAnalyzer.views.android.playstore import (
    get_app_details,
)
from mobsf.StaticAnalyzer.views.android.strings import (
    get_strings_metadata,
)
from mobsf.StaticAnalyzer.views.common.shared_func import (
    get_avg_cvss,
    hash_gen,
    unzip,
)
from mobsf.StaticAnalyzer.views.common.firebase import (
    firebase_analysis,
)
from mobsf.StaticAnalyzer.views.common.appsec import (
    get_android_dashboard,
)
from mobsf.StaticAnalyzer.views.common.async_task import (
    async_analysis,
    mark_task_completed,
    mark_task_started,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    has_permission,
)

logger = logging.getLogger(__name__)


def initialize_app_dic(app_dic, file_ext):
    checksum = app_dic['md5']
    app_dic['app_file'] = f'{checksum}.{file_ext}'
    app_dic['app_path'] = (app_dic['app_dir'] / app_dic['app_file']).as_posix()
    app_dic['app_dir'] = app_dic['app_dir'].as_posix() + '/'
    return checksum


def get_size_and_hashes(app_dic):
    app_dic['size'] = str(file_size(app_dic['app_path'])) + 'MB'
    app_dic['sha1'], app_dic['sha256'] = hash_gen(app_dic['md5'], app_dic['app_path'])


def get_manifest_data(app_dic):
    """Get Manifest Data."""
    # Manifest XML parsed
    get_parsed_manifest(app_dic)
    # Populates manifest_file, manifest_namespace, manifest_parsed_xml

    # Manifest data extraction
    man_data_dic = extract_manifest_data(app_dic)
    # Manifest Analysis
    man_analysis = manifest_analysis(app_dic, man_data_dic)
    return man_data_dic, man_analysis


def print_scan_subject(app_dic, man_data):
    """Log scan subject."""
    checksum = app_dic['md5']
    app_name = app_dic.get('real_name')
    pkg_name = man_data.get('packagename')
    pkg_name2 = app_dic.get('apk_features', {}).get('package')
    if not pkg_name:
        pkg_name = pkg_name2
    subject = 'Android App'
    if app_name and pkg_name:
        subject = f'{app_name} ({pkg_name})'
    elif pkg_name:
        subject = pkg_name
    elif app_name:
        subject = app_name
    msg = f'Performing Static Analysis on: {subject}'
    logger.info(msg)
    append_scan_status(checksum, msg)
    if subject == 'Failed':
        subject = f'({subject})'
    app_dic['subject'] = subject


def clean_up(app_dic):
    """Clean up for pickling."""
    app_dic['androguard_apk'] = None
    app_dic['androguard_apk_resources'] = None


def apk_analysis_task(checksum, app_dic, rescan, queue=False):
    """APK Analysis Task."""
    context = None
    try:
        if queue:
            settings.ASYNC_ANALYSIS = True
            mark_task_started(checksum)
        append_scan_status(checksum, 'init')
        get_size_and_hashes(app_dic)
        msg = 'Extracting APK'
        logger.info(msg)
        append_scan_status(checksum, msg)
        app_dic['zipped'] = 'apk'
        # Extract APK and get files
        app_dic['files'] = unzip(
            checksum,
            app_dic['app_path'],
            app_dic['app_dir'])
        # Extract APK data with Androguard
        androguard_parse(app_dic)
        # Populates androguard_apk, androguard_manifest_xml, androguard_string_resources
        # Extract APK data with AAPT/AAPT2
        aapt_parse(app_dic)  # Populates apk_features, files, apk_strings
        get_hardcoded_cert_keystore(app_dic)  # Populates file_analysis
        # Manifest Data
        man_data_dic, man_analysis = get_manifest_data(app_dic)
        # Get App name
        get_apk_name(app_dic)  # Populates real_name
        print_scan_subject(app_dic, man_data_dic)  # Populate subject
        get_app_details(app_dic, man_data_dic)  # Populates playstore
        # Malware Permission check
        mal_perms = permissions.check_malware_permission(
            checksum,
            man_data_dic['perm'])
        man_analysis['malware_permissions'] = mal_perms
        # Get icon
        get_icon_apk(app_dic)  # Populates icon_path
        elf_dict = library_analysis(
            checksum,
            app_dic['app_dir'],
            'elf')
        cert_dic = cert_info(app_dic, man_data_dic)
        apkid_results = apkid.apkid_analysis(
            checksum,
            app_dic['app_path'])
        trackers = Trackers.Trackers(
            checksum,
            app_dic['app_dir'],
            app_dic['tools_dir']).get_trackers()
        apk_2_java(
            checksum,
            app_dic['app_path'],
            app_dic['app_dir'],
            settings.DOWNLOADED_TOOLS_DIR)
        dex_2_smali(
            checksum,
            app_dic['app_dir'],
            app_dic['tools_dir'])
        code_an_dic = code_analysis(
            checksum,
            app_dic['app_dir'],
            app_dic['zipped'],
            app_dic['manifest_file'],
            man_data_dic['perm'])
        # Get the strings and metadata
        get_strings_metadata(
            app_dic,
            elf_dict['elf_strings'],
            ['.java'],
            code_an_dic)
        # Firebase DB Check
        code_an_dic['firebase'] = firebase_analysis(
            checksum,
            code_an_dic)
        # Domain Extraction and Malware Check
        code_an_dic['domains'] = MalwareDomainCheck().scan(
            checksum,
            code_an_dic['urls_list'])
        context = save_get_ctx(
            app_dic,
            man_data_dic,
            man_analysis,
            code_an_dic,
            cert_dic,
            elf_dict['elf_analysis'],
            apkid_results,
            trackers,
            rescan,
        )
        if queue:
            return mark_task_completed(
                checksum, app_dic['subject'], 'Success')
        return context, None
    except Exception as exp:
        if queue:
            return mark_task_completed(
                checksum, 'Failed', repr(exp))
        return context, repr(exp)
    finally:
        # Clean up
        clean_up(app_dic)


def generate_dynamic_context(request, app_dic, checksum, context, api):
    """Generate Dynamic Context."""
    context['appsec'] = get_android_dashboard(context, True)
    context['average_cvss'] = get_avg_cvss(context['code_analysis']['findings'])
    logcat_file = Path(app_dic['app_dir']) / 'logcat.txt'
    context['dynamic_analysis_done'] = logcat_file.exists()
    context['virus_total'] = None
    if settings.VT_ENABLED:
        vt = VirusTotal.VirusTotal(checksum)
        context['virus_total'] = vt.get_result(app_dic['app_path'])
    template = 'static_analysis/android_binary_analysis.html'
    return context if api else render(request, template, context)


def apk_analysis(request, app_dic, rescan, api):
    """APK Analysis."""
    checksum = initialize_app_dic(app_dic, 'apk')
    db_entry = StaticAnalyzerAndroid.objects.filter(MD5=checksum)
    if db_entry.exists() and not rescan:
        context = get_context_from_db_entry(db_entry)
        return generate_dynamic_context(request, app_dic, checksum, context, api)
    else:
        # APK Analysis
        if not has_permission(request, Permissions.SCAN, api):
            return print_n_send_error_response(request, 'Permission Denied', False)
        if settings.ASYNC_ANALYSIS:
            return async_analysis(
                checksum,
                api,
                app_dic.get('app_name', ''),
                apk_analysis_task, checksum, app_dic, rescan)
        context, err = apk_analysis_task(checksum, app_dic, rescan)
        if err:
            return print_n_send_error_response(request, err, api)
        return generate_dynamic_context(request, app_dic, checksum, context, api)


def src_analysis_task(checksum, app_dic, rescan, pro_type, queue=False):
    """Android ZIP Source Code Analysis Begins."""
    context = None
    try:
        if queue:
            settings.ASYNC_ANALYSIS = True
            mark_task_started(checksum)
        cert_dic = {
            'certificate_info': '',
            'certificate_status': '',
            'description': '',
        }
        app_dic['strings'] = []
        app_dic['secrets'] = []
        # Above fields are only available for APK and not ZIP
        app_dic['zipped'] = pro_type
        get_hardcoded_cert_keystore(app_dic)
        # Manifest Data
        man_data_dic, man_analysis = get_manifest_data(app_dic)
        # Get app name
        get_apk_name(app_dic)
        # Print scan subject
        print_scan_subject(app_dic, man_data_dic)
        get_app_details(app_dic, man_data_dic)
        # Malware Permission check
        mal_perms = permissions.check_malware_permission(
            checksum,
            man_data_dic['perm'])
        man_analysis['malware_permissions'] = mal_perms
        # Get icon
        get_icon_from_src(
            app_dic,
            man_data_dic['icons'])
        code_an_dic = code_analysis(
            checksum,
            app_dic['app_dir'],
            app_dic['zipped'],
            app_dic['manifest_file'],
            man_data_dic['perm'])
        # Get the strings and metadata
        get_strings_metadata(
            app_dic,
            None,
            ['.java', '.kt'],
            code_an_dic)
        # Firebase DB Check
        code_an_dic['firebase'] = firebase_analysis(
            checksum,
            code_an_dic)
        # Domain Extraction and Malware Check
        code_an_dic['domains'] = MalwareDomainCheck().scan(
            checksum,
            code_an_dic['urls_list'])
        # Extract Trackers from Domains
        trackers = Trackers.Trackers(
            checksum,
            None,
            app_dic['tools_dir']).get_trackers_domains_or_deps(
                code_an_dic['domains'], [])
        context = save_get_ctx(
            app_dic,
            man_data_dic,
            man_analysis,
            code_an_dic,
            cert_dic,
            [],
            {},
            trackers,
            rescan,
        )
        if queue:
            return mark_task_completed(
                checksum, app_dic['subject'], 'Success')
    except Exception as exp:
        if queue:
            return mark_task_completed(
                checksum, 'Failed', repr(exp))
    return context


def generate_dynamic_src_context(request, context, api):
    """Generate Dynamic Source Context."""
    context['appsec'] = get_android_dashboard(context, True)
    context['average_cvss'] = get_avg_cvss(context['code_analysis']['findings'])
    template = 'static_analysis/android_source_analysis.html'
    return context if api else render(request, template, context)


def src_analysis(request, app_dic, rescan, api):
    """Source Code Analysis."""
    checksum = initialize_app_dic(app_dic, 'zip')
    ret = f'/static_analyzer_ios/{checksum}/'
    db_entry = StaticAnalyzerAndroid.objects.filter(
        MD5=checksum)
    ios_db_entry = StaticAnalyzerIOS.objects.filter(
        MD5=checksum)
    if db_entry.exists() and not rescan:
        context = get_context_from_db_entry(db_entry)
        return generate_dynamic_src_context(request, context, api)
    elif ios_db_entry.exists() and not rescan:
        return {'type': 'ios'} if api else HttpResponseRedirect(ret)
    else:
        # Initialize for both Android and iOS Source Analysis
        append_scan_status(checksum, 'init')
        get_size_and_hashes(app_dic)
        msg = 'Extracting ZIP'
        logger.info(msg)
        append_scan_status(checksum, msg)
        app_dic['files'] = unzip(
            checksum,
            app_dic['app_path'],
            app_dic['app_dir'])
        # Check if Valid Directory Structure and get ZIP Type
        pro_type, valid = valid_source_code(
            checksum,
            app_dic['app_dir'])
        msg = f'Source code type - {pro_type}'
        logger.info(msg)
        append_scan_status(checksum, msg)
        # Handle iOS Source Code
        if valid and pro_type == 'ios':
            msg = 'Redirecting to iOS Source Code Analyzer'
            logger.info(msg)
            append_scan_status(checksum, msg)
            ret = f'{ret}?rescan={str(int(rescan))}'
            return {'type': 'ios'} if api else HttpResponseRedirect(ret)
        # Android Source Code Analysis
        if not has_permission(request, Permissions.SCAN, api):
            return print_n_send_error_response(
                request,
                'Permission Denied',
                False)
        if valid and (pro_type in ['eclipse', 'studio']):
            if settings.ASYNC_ANALYSIS:
                return async_analysis(
                    checksum,
                    api,
                    app_dic.get('app_name', ''),
                    src_analysis_task, checksum, app_dic, rescan, pro_type)
            context = src_analysis_task(checksum, app_dic, rescan, pro_type)
            return generate_dynamic_src_context(request, context, api)
        else:
            msg = 'This ZIP Format is not supported'
            if api:
                return print_n_send_error_response(
                    request,
                    msg,
                    True)
            else:
                print_n_send_error_response(request, msg, False)
                ctx = {
                    'title': 'Invalid ZIP archive',
                    'version': settings.MOBSF_VER,
                }
                template = 'general/zip.html'
                return render(request, template, ctx)


def is_android_source(app_path):
    """Detect Android Source and IDE Type."""
    # Eclipse
    man = app_path / 'AndroidManifest.xml'
    src = app_path / 'src'
    if man.is_file() and src.exists():
        return 'eclipse', True

    # Studio
    man = app_path / 'app' / 'src' / 'main' / 'AndroidManifest.xml'
    java = app_path / 'app' / 'src' / 'main' / 'java'
    kotlin = app_path / 'app' / 'src' / 'main' / 'kotlin'
    if man.is_file() and (java.exists() or kotlin.exists()):
        return 'studio', True

    return None, False


def move_to_parent(inside_path, app_path):
    """Move contents of inside to app dir."""
    for item in inside_path.iterdir():
        shutil.move(str(item), str(app_path))
    shutil.rmtree(inside_path)


def valid_source_code(checksum, app_dir):
    """Test if this is a valid source code zip."""
    try:
        msg = 'Detecting source code type'
        logger.info(msg)
        append_scan_status(checksum, msg)

        app_path = Path(app_dir)
        ide, is_and = is_android_source(app_path)

        if ide:
            return ide, is_and

        # Relaxed Android Source check, one level down
        for subdir in app_path.iterdir():
            if subdir.is_dir() and subdir.exists():
                ide, is_and = is_android_source(subdir)
                if ide:
                    move_to_parent(subdir, app_path)
                    return ide, is_and

        # iOS Source
        xcode = [f for f in app_path.iterdir() if f.suffix == '.xcodeproj']
        if xcode:
            return 'ios', True

        # Relaxed iOS Source Check
        for subdir in app_path.iterdir():
            if subdir.is_dir() and subdir.exists():
                if any(f.suffix == '.xcodeproj' for f in subdir.iterdir()):
                    return 'ios', True

        return '', False
    except Exception as exp:
        msg = 'Error identifying source code type from zip'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))



# -*- coding: utf_8 -*-
"""iOS Dynamic Analysis."""
import logging
import json
from pathlib import Path

from mobsf.DynamicAnalyzer.views.common.shared import (
    extract_urls_domains_emails,
    get_app_files,
)

logger = logging.getLogger(__name__)


def get_screenshots(checksum, download_dir):
    """Get Screenshots."""
    screenshots = []
    screen_dir = Path(download_dir)
    for img in screen_dir.glob('*.png'):
        if (img.name.startswith(checksum)
                and 'sshot' in img.name):
            screenshots.append(img.name)
    return screenshots


def get_logs_data(app_dir, bundle_id):
    """Get Data for analysis."""
    data = []
    dump_file = Path(app_dir) / 'mobsf_dump_file.txt'
    fd_log_file = Path(app_dir) / 'mobsf_frida_out.txt'
    flows = Path.home() / '.httptools' / 'flows'
    web_file = flows / f'{bundle_id}.flow.txt'
    if dump_file.exists():
        data.append(dump_file.read_text('utf-8', 'ignore'))
    if fd_log_file.exists():
        data.append(fd_log_file.read_text('utf-8', 'ignore'))
    if web_file.exists():
        data.append(web_file.read_text('utf-8', 'ignore'))
    return '\n'.join(data)


def run_analysis(app_dir, bundle_id, checksum):
    """Run Dynamic File Analysis."""
    analysis_result = {}
    logger.info('Dynamic File Analysis')
    domains = {}
    # Collect Log data
    data = get_logs_data(app_dir, bundle_id)
    urls, domains, emails = extract_urls_domains_emails(
        checksum,
        data)
    # App data files analysis
    pfiles = get_app_files(app_dir, f'{checksum}-app-container')
    analysis_result['sqlite'] = pfiles['sqlite']
    analysis_result['plist'] = pfiles['plist']
    analysis_result['others'] = pfiles['others']
    analysis_result['urls'] = urls
    analysis_result['domains'] = domains
    analysis_result['emails'] = list(emails)
    return analysis_result


def ios_api_analysis(app_dir):
    """The iOS API Analysis."""
    dump = {
        'cookies': [],
        'crypto': [],
        'network': [],
        'files': [],
        'keychain': [],
        'logs': [],
        'credentials': [],
        'userdefaults': {},
        'pasteboard': [],
        'textinputs': [],
        'datadir': [],
        'sql': [],
        'json': [],
    }
    try:
        dump_file = app_dir / 'mobsf_dump_file.txt'
        if not dump_file.exists():
            return dump
        logger.info('Analyzing Frida Data Dump')
        data = dump_file.read_text(
            encoding='utf-8',
            errors='ignore').splitlines()
        for line in data:
            parsed = json.loads(line)
            if parsed.get('cookies'):
                dump['cookies'] = parsed['cookies']
            elif parsed.get('crypto'):
                dump['crypto'].append(parsed['crypto'])
            elif parsed.get('filename'):
                dump['files'].append(parsed['filename'])
            elif parsed.get('keychain'):
                dump['keychain'] = parsed['keychain']
            elif parsed.get('nslog'):
                dump['logs'].append(parsed['nslog'])
            elif parsed.get('credentialstorage'):
                dump['credentials'] = parsed['credentialstorage']
            elif parsed.get('nsuserdefaults'):
                dump['userdefaults'] = parsed['nsuserdefaults']
            elif (parsed.get('pasteboard')
                    and parsed.get('pasteboard') not in dump['pasteboard']):
                dump['pasteboard'].append(parsed['pasteboard'])
            elif parsed.get('textinput'):
                dump['textinputs'].append(parsed['textinput'])
            elif parsed.get('network'):
                dump['network'].append(parsed['network'])
            elif parsed.get('datadir'):
                dump['datadir'] = parsed['datadir']
            elif parsed.get('sql'):
                dump['sql'].append(parsed['sql'])
            elif parsed.get('json'):
                dump['json'].append(parsed['json'])
            dump['files'] = list(set(dump['files']))
            dump['logs'] = list(set(dump['logs']))
            if len(dump['network']) > 0:
                dump['network'] = list(
                    {v['url']: v for v in dump['network']}.values())
    except Exception:
        logger.exception('Analyzing dump data failed')
    return dump



#!/usr/bin/env python3
"""Django Manage."""
import warnings
import os
import sys

warnings.filterwarnings('ignore', category=UserWarning, module='cffi')

if __name__ == '__main__':
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'mobsf.MobSF.settings')

    from django.core.management import execute_from_command_line
    if 'runserver' in sys.argv:
        print('We do not allow debug server anymore. '
              'Please follow official docs: '
              'https://mobsf.github.io/docs/')
        sys.exit(0)
    execute_from_command_line(sys.argv)



import logging
from pathlib import Path

import lief

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    settings_enabled,
)
from mobsf.StaticAnalyzer.views.common.binary.elf import (
    ELFChecksec,
)
from mobsf.StaticAnalyzer.views.common.binary.macho import (
    MachOChecksec,
)


logger = logging.getLogger(__name__)


def library_analysis(checksum, src, arch):
    """Perform library binary analysis."""
    base_dir = Path(settings.UPLD_DIR) / checksum
    res = {
        f'{arch}_analysis': [],
        f'{arch}_strings': [],
        f'{arch}_symbols': [],
        'framework_analysis': [],
        'framework_strings': [],
        'framework_symbols': [],
    }
    try:
        if arch == 'macho':
            analysis = MachOChecksec
            ext = '*.dylib'
            if not settings_enabled('DYLIB_ANALYSIS_ENABLED'):
                return res
        elif arch == 'elf':
            analysis = ELFChecksec
            ext = '*.so'
            if not settings_enabled('SO_ANALYSIS_ENABLED'):
                return res
        elif arch == 'ar':
            ext = '*.o'
            res[f'{arch}_a'] = ''
        msg = 'Library Binary Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)
        # Supports Static Library, Shared objects, Dynamic Library,
        # from APK, SO, AAR, JAR, IPA, DYLIB, and A
        for libfile in Path(src).rglob(ext):
            if '__MACOSX' in libfile.as_posix():
                continue
            rel_path = libfile.relative_to(base_dir).as_posix()
            msg = f'Analyzing {rel_path}'
            logger.info(msg)
            append_scan_status(checksum, msg)
            if arch == 'ar':
                # Handle static library
                if lief.is_macho(libfile.as_posix()):
                    analysis = MachOChecksec
                    res[f'{arch}_a'] = 'MachO'
                elif lief.is_elf(libfile.as_posix()):
                    analysis = ELFChecksec
                    res[f'{arch}_a'] = 'ELF'
                else:
                    continue
            chk = analysis(libfile, rel_path)
            chksec = chk.checksec()
            strings = chk.strings()
            symbols = chk.get_symbols()

            if chksec:
                res[f'{arch}_analysis'].append(chksec)
            if strings:
                res[f'{arch}_strings'].append({
                    rel_path: strings})
            if symbols:
                res[f'{arch}_symbols'].append({
                    rel_path: symbols})
        if ext == '*.dylib':
            # Do Framework Analysis for iOS
            frameworks_analysis(checksum, src, base_dir, res)
            if res['framework_strings']:
                res[f'{arch}_strings'].extend(
                    res['framework_strings'])
    except Exception as exp:
        msg = 'Error Performing Library Binary Analysis'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return res


def frameworks_analysis(checksum, src, base_dir, res):
    """Binary Analysis on Frameworks."""
    try:
        msg = 'Framework Binary Analysis Started'
        logger.info(msg)
        append_scan_status(checksum, msg)
        # Supports iOS Frameworks
        for ffile in Path(src).rglob('*'):
            parent = ffile.parents[0].name
            if not parent.endswith('.framework'):
                continue
            rel_path = ffile.relative_to(base_dir).as_posix()
            if ffile.suffix != '' or ffile.name not in parent:
                continue
            # Frameworks/XXX.framework/XXX
            msg = f'Analyzing {rel_path}'
            logger.info(msg)
            append_scan_status(checksum, msg)
            chk = MachOChecksec(ffile, rel_path)
            chksec = chk.checksec()
            strings = chk.strings()
            symbols = chk.get_symbols()
            if chksec:
                res['framework_analysis'].append(chksec)
            if strings:
                res['framework_strings'].append({
                    rel_path: strings})
            if symbols:
                res['framework_symbols'].append({
                    rel_path: symbols})
    except Exception as exp:
        msg = 'Error Performing Framework Binary Analysis'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))



# -*- coding: utf_8 -*-
# Module for Malware Analysis
import io
import logging
import os
import re
from pathlib import Path
from socket import (
    gaierror,
    gethostbyname,
)
from urllib.parse import urlparse

from django.conf import settings

import IP2Location

from mobsf.MobSF.utils import (
    append_scan_status,
    is_internet_available,
    settings_enabled,
    update_local_db,
)

logger = logging.getLogger(__name__)


class MalwareDomainCheck:

    def __init__(self):
        self.sig_dir = Path(settings.SIGNATURE_DIR)
        self.malwaredomainlist = self.sig_dir / 'malwaredomainlist'
        self.maltrail = self.sig_dir / 'maltrail-malware-domains.txt'
        self.iplocbin = self.sig_dir / 'IP2LOCATION-LITE-DB5.IPV6.BIN'
        self.result = {}
        self.domainlist = None
        self.IP2Loc = IP2Location.IP2Location()

    def update_maltrail_db(self):
        """Check for update in maltrail DB."""
        try:
            mal_db = self.maltrail
            resp = update_local_db(
                'Maltrail',
                settings.MALTRAIL_DB_URL, mal_db)
            if not resp:
                return
            # DB needs update
            # Check2: DB Syntax Changed
            lines = resp.decode('utf-8', 'ignore').splitlines()
            if len(lines) > 100:
                logger.info('Updating Maltrail Database')
                with open(mal_db, 'wb') as wfp:
                    wfp.write(resp)
            else:
                logger.warning('Unable to Update Maltrail DB')
        except Exception:
            logger.exception('[ERROR] Maltrail DB Update')

    def gelocation(self):
        """Perform Geolocation."""
        try:
            ofac_list = {
                'cuba', 'iran', 'north korea',
                'russia', 'syria', 'balkans',
                'belarus', 'myanmar', 'congo',
                'ethiopia', 'hong kong', 'iraq',
                'lebanon', 'libya', 'sudan',
                'venezuela', 'yemen', 'zimbabwe',
                'crimea', 'donetsk', 'luhansk',
                'afghanistan', 'china', 'ivory coast',
                'cyprus', 'eritrea', 'haiti',
                'liberia', 'somalia', 'sri lanka',
                'vietnam', 'south sudan',
            }
            self.IP2Loc.open(self.iplocbin)
            for domain in self.domainlist:
                # Tag Good Domains
                if domain not in self.result:
                    tmp_d = {}
                    tmp_d['bad'] = 'no'
                    self.result[domain] = tmp_d
                # GeoIP
                ip = None
                try:
                    ip = gethostbyname(domain)
                except (gaierror, UnicodeError):
                    pass
                if ip:
                    rec = self.IP2Loc.get_all(ip)
                    self.result[domain]['geolocation'] = rec.__dict__
                    country = rec.__dict__.get('country_long')
                    region = rec.__dict__.get('region')
                    city = rec.__dict__.get('city')
                    self.result[domain]['ofac'] = False
                    if country and country.lower() in ofac_list:
                        self.result[domain]['ofac'] = True
                    elif region and region.lower() in ofac_list:
                        self.result[domain]['ofac'] = True
                    elif city and city.lower() in ofac_list:
                        self.result[domain]['ofac'] = True
                else:
                    self.result[domain]['geolocation'] = None
        except Exception:
            logger.exception('Failed to Perform Geolocation')
        finally:
            if self.IP2Loc:
                self.IP2Loc.close()

    def malware_check(self):
        """Check domains against malware database."""
        try:
            mal_db = self.malwaredomainlist

            if not mal_db.exists():
                logger.warning('Malware domain list not found: %s', mal_db)
                return

            if not os.access(mal_db, os.R_OK):
                logger.error('Insufficient permissions to read: %s', mal_db)
                return

            with io.open(mal_db,
                         mode='r',
                         encoding='utf8',
                         errors='ignore') as flip:
                entry_list = flip.readlines()

            if not entry_list:
                logger.warning('Malware domain database is empty')
                return

            for entry in entry_list:
                enlist = entry.split('","')
                if len(enlist) > 5:
                    details_dict = {}
                    details_dict['domain_or_url'] = enlist[1]
                    details_dict['ip'] = enlist[2]
                    details_dict['desc'] = enlist[4]
                    details_dict['bad'] = 'yes'
                    dmn_url = details_dict['domain_or_url']
                    for domain in self.domainlist:
                        dmn_neturl = get_netloc(dmn_url)
                        if (((dmn_neturl == domain or dmn_neturl == domain[4:])
                                and (len(dmn_url) > 1))
                                or details_dict['ip'].startswith(domain)):
                            self.result[domain] = details_dict
        except FileNotFoundError:
            logger.error('Malware domain database file not found: %s', mal_db)
        except PermissionError:
            logger.error('Permission denied accessing malware database: %s', mal_db)
        except IOError:
            logger.exception('I/O error reading malware database')
        except Exception:
            logger.exception('[ERROR] Performing Malware check')

    def maltrail_check(self):
        """Check domains against maltrail database."""
        try:
            mal_db = self.maltrail

            if not mal_db.exists():
                logger.warning('Maltrail database not found: %s', mal_db)
                return

            if not os.access(mal_db, os.R_OK):
                logger.error('Insufficient permissions to read: %s', mal_db)
                return

            with io.open(mal_db,
                         mode='r',
                         encoding='utf8',
                         errors='ignore') as flip:
                entry_list = flip.read().splitlines()

            if not entry_list:
                logger.warning('Maltrail database is empty')
                return

            for domain in self.domainlist:
                if domain in entry_list:
                    self.result[domain] = {
                        'domain_or_url': domain,
                        'ip': 'N/A',
                        'desc': 'Malicious Domain tagged by Maltrail',
                        'bad': 'yes',
                    }
        except FileNotFoundError:
            logger.error('Maltrail database file not found: %s', mal_db)
        except PermissionError:
            logger.error('Permission denied accessing maltrail database: %s', mal_db)
        except IOError:
            logger.exception('I/O error reading maltrail database')
        except Exception:
            logger.exception('[ERROR] Performing Maltrail Check')

    def update(self):
        """Update malware databases with connection checks."""
        try:
            if is_internet_available():
                logger.info('Checking for malware database updates')
                self.update_maltrail_db()
            else:
                logger.warning('Internet not available. '
                               'Skipping Malware Database Update.')
        except Exception:
            logger.exception('Failed to update malware databases')

    def scan(self, checksum, urls):
        """Scan method."""
        if not settings_enabled('DOMAIN_MALWARE_SCAN'):
            logger.info('Domain Malware check disabled in settings')
            return self.result

        msg = 'Performing Malware check on extracted domains'
        urls = [u.lower() for u in urls]
        append_scan_status(checksum, msg)

        try:
            self.domainlist = get_domains(urls)
            if not self.domainlist:
                logger.info('No domains found to analyze')
                return self.result

            logger.info('Analyzing %d domains', len(self.domainlist))
            self.update()
            self.malware_check()
            self.maltrail_check()
            self.gelocation()

            # Log analysis results
            malicious_count = sum(1 for domain in self.result.values()
                                  if domain.get('bad') == 'yes')
            logger.info(
                'Malware analysis completed: %d malicious domains found',
                malicious_count)

        except Exception as exp:
            logger.exception('Critical error during domain malware check')
            append_scan_status(checksum, 'Domain malware check failed', repr(exp))

        return self.result


# Helper Functions

def verify_domain(checkeddom):
    """Verify domain."""
    try:
        if not checkeddom:
            return False

        if (len(checkeddom) > 2
                and '.' in checkeddom
                and (checkeddom.endswith('.') is False
                     and re.search('[a-zA-Z0-9]', checkeddom))):
            return True
        else:
            return False
    except Exception:
        logger.exception('[ERROR] Verifying Domain')
        return False


def get_netloc(url):
    """Get Domain."""
    try:
        domain = ''
        parse_uri = urlparse(url)
        if not parse_uri.scheme:
            url = '//' + url
            parse_uri = urlparse(url)
        domain = '{uri.netloc}'.format(uri=parse_uri)
        if verify_domain(domain):
            return domain
    except Exception:
        logger.exception('[ERROR] Extracting Domain form URL')
    return None


def sanitize_domain(domain):
    """Sanitize domain to be RFC1034 compliant."""
    try:
        if not domain:
            return domain

        domain = domain.split('_')[0]
        domain = re.sub(r'[^\w^\.^\-]', '', domain)
        if domain.startswith('-'):
            domain = sanitize_domain(domain[1:])
        elif domain.endswith('-'):
            domain = sanitize_domain(domain[:-1])
    except Exception:
        logger.exception('[ERROR] Sanitizing domain')
    return domain


def get_domains(urls):
    """Get Domains."""
    domains = set()
    try:
        if not urls:
            return domains

        for url in urls:
            if not url:
                continue

            parse_uri = urlparse(url)
            if not parse_uri.scheme:
                url = '//' + url
                parse_uri = urlparse(url)
            domain = sanitize_domain(
                '{uri.hostname}'.format(uri=parse_uri))
            if verify_domain(domain):
                domains.add(domain)
    except Exception:
        logger.exception('[ERROR] Extracting Domain form URL')
    return domains



# -*- coding: utf_8 -*-
import logging

import requests

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    upstream_proxy,
)

logger = logging.getLogger(__name__)


def app_search(checksum, app_id):
    """IOS Get App Details from App Store."""
    msg = f'Fetching Details from App Store: {app_id}'
    logger.info(msg)
    append_scan_status(checksum, msg)
    lookup_url = settings.ITUNES_URL
    req_url = '{}?bundleId={}&country={}&entity=software'.format(
        lookup_url, app_id, 'us')
    headers = {
        'User-Agent': ('Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) '
                       'AppleWebKit/537.36 (KHTML, like Gecko) '
                       'Chrome/39.0.2171.95 Safari/537.36')}
    try:
        det = {}
        proxies, verify = upstream_proxy('https')
        req = requests.get(
            req_url,
            timeout=5,
            headers=headers,
            proxies=proxies,
            verify=verify)
        resp = req.json()
        if resp['results']:
            det = resp['results'][0]
            return {
                'features': det['features'] or [],
                'icon': (det['artworkUrl512']
                         or det['artworkUrl100']
                         or det['artworkUrl60'] or ''),
                'developer_id': det['artistId'],
                'developer': det['artistName'],
                'developer_url': det['artistViewUrl'],
                'developer_website': det['sellerUrl'],
                'supported_devices': det['supportedDevices'],
                'title': det['trackName'],
                'app_id': det['bundleId'],
                'category': det['genres'] or [],
                'description': det['description'],
                'price': det['price'],
                'itunes_url': det['trackViewUrl'],
                'score': det['averageUserRating'],
                'error': False,
            }
        logger.warning('Unable to get app details.')
        return {'error': True}
    except Exception as exp:
        msg = 'Failed to get app details'
        logger.warning(msg)
        append_scan_status(checksum, msg, repr(exp))
        return {'error': True}



import logging
import sys
import shutil
import tempfile
import zipfile
import platform
import os
import ssl
from pathlib import Path
from urllib.request import (
    HTTPSHandler,
    ProxyHandler,
    Request,
    build_opener,
    getproxies,
)

from mobsf.MobSF.exceptions import PathTraversalError

logging.basicConfig(
    level=logging.INFO,
    format='[%(levelname)s] %(asctime)-15s - %(message)s',
    datefmt='%d/%b/%Y %H:%M:%S')
logger = logging.getLogger(__name__)


def standalone_upstream_proxy():
    """Set upstream Proxy for urllib - standalone."""
    upstream_proxy_enabled = bool(os.getenv('MOBSF_UPSTREAM_PROXY_ENABLED', ''))

    if upstream_proxy_enabled:
        upstream_proxy_username = os.getenv('MOBSF_UPSTREAM_PROXY_USERNAME', '')
        upstream_proxy_password = os.getenv('MOBSF_UPSTREAM_PROXY_PASSWORD', '')
        upstream_proxy_type = os.getenv('MOBSF_UPSTREAM_PROXY_TYPE', 'http')
        upstream_proxy_ip = os.getenv('MOBSF_UPSTREAM_PROXY_IP', '127.0.0.1')
        upstream_proxy_port = int(os.getenv('MOBSF_UPSTREAM_PROXY_PORT', '3128'))

        # Handle Docker proxy IP translation
        if os.getenv('MOBSF_PLATFORM') == 'docker':
            if (upstream_proxy_ip and upstream_proxy_ip.strip() in
                    ('127.0.0.1', 'localhost')):
                upstream_proxy_ip = 'host.docker.internal'

        if not upstream_proxy_username:
            proxy_port = str(upstream_proxy_port)
            proxy_host = f'{upstream_proxy_type}://{upstream_proxy_ip}:{proxy_port}'
        else:
            proxy_port = str(upstream_proxy_port)
            proxy_host = (f'{upstream_proxy_type}://{upstream_proxy_username}:'
                          f'{upstream_proxy_password}@{upstream_proxy_ip}:'
                          f'{proxy_port}')

        # For urllib, we need to set both http and https proxies
        proxies = {
            'http': proxy_host,
            'https': proxy_host,
        }
    else:
        proxies = {}

    upstream_proxy_ssl_verify = os.getenv('MOBSF_UPSTREAM_PROXY_SSL_VERIFY', '1')
    verify = upstream_proxy_ssl_verify in ('1', '"1"')
    return proxies, verify


def download_file(url, file_path):
    req = Request(url)

    # Check for system proxies first (http_proxy, https_proxy env vars)
    system_proxies = getproxies()

    if system_proxies:
        proxies = system_proxies
        verify = True  # Default to verify for system proxies
        logger.info('Using system proxies (SSL verify: %s)', verify)
    else:
        # Check if MobSF upstream proxy is explicitly configured
        upstream_proxy_enabled = bool(os.getenv('MOBSF_UPSTREAM_PROXY_ENABLED', ''))

        if upstream_proxy_enabled:
            proxies, verify = standalone_upstream_proxy()
            logger.info('Using MobSF upstream proxies (SSL verify: %s)', verify)
        else:
            # No proxy configuration - use direct connection
            proxies = {}
            verify = True

    proxy_handler = ProxyHandler(proxies)

    if verify:
        ssl_context = ssl.create_default_context()
    else:
        ssl_context = ssl._create_unverified_context()

    https_handler = HTTPSHandler(context=ssl_context)
    opener = build_opener(proxy_handler, https_handler)

    with opener.open(req) as response:
        if response.status == 200:
            file_size = int(response.headers.get('Content-Length', 0))
            downloaded = 0
            block_size = 8192  # 8KB

            with open(file_path, 'wb') as f:
                while True:
                    buffer = response.read(block_size)
                    if not buffer:
                        break
                    downloaded += len(buffer)
                    f.write(buffer)

                    # Print progress
                    if file_size > 0:
                        done = int(50 * downloaded / file_size)
                        fmt = (f'\r[{"#" * done}{"-" * (50 - done)}] '
                               f'{downloaded * 100 / file_size:.2f}%')
                        sys.stdout.write(fmt)
                        sys.stdout.flush()

            if downloaded != file_size:
                err = (f'Downloaded file size ({downloaded}) '
                       f'does not match expected size ({file_size})')
                raise Exception(err)

            return downloaded
        else:
            raise Exception(f'Failed to download file. Status code: {response.status}')


def install_jadx(mobsf_home, version='1.5.0'):
    """Install JADX dynamically."""
    try:
        url = ('https://github.com/skylot/jadx/releases/download/'
               f'v{version}/jadx-{version}.zip')
        jadx_dir = Path(mobsf_home) / 'tools' / 'jadx'
        extract_dir = jadx_dir / f'jadx-{version}'

        if extract_dir.exists():
            logger.info('JADX is already installed at %s', extract_dir)
            return

        logger.info('Downloading JADX from %s', url)
        shutil.rmtree(jadx_dir, ignore_errors=True)

        with tempfile.NamedTemporaryFile(
                delete=False,
                mode='wb',
                suffix='.zip') as tmp_zip_file:

            downloaded_size = download_file(url, tmp_zip_file.name)
            logger.info('JADX download complete. File size: %d bytes', downloaded_size)

            # Extract the zip file
            logger.info('Extracting JADX to %s', extract_dir)
            extract_dir.mkdir(parents=True, exist_ok=True)
            with zipfile.ZipFile(tmp_zip_file.name, 'r') as zip_ref:
                abs_extract_dir = os.path.realpath(extract_dir)
                for member in zip_ref.namelist():
                    member_path = os.path.realpath(
                        os.path.join(abs_extract_dir, member))
                    if not (member_path.startswith(abs_extract_dir + os.sep)
                            or member_path == abs_extract_dir):
                        raise PathTraversalError('Attempted Path Traversal in Zip File')
                    zip_ref.extract(member, extract_dir)

        # Set execute permission
        set_rwxr_xr_x_permission_recursively(extract_dir)

        logger.info('JADX installed successfully')
    except Exception:
        logger.exception('Error during JADX installation')
    finally:
        if 'tmp_zip_file' in locals():
            Path(tmp_zip_file.name).unlink()


def set_rwxr_xr_x_permission_recursively(directory_path):
    """Set execute permissions recursively."""
    if platform.system() == 'Windows':
        logger.info('Permission setting is skipped on non-Unix systems.')
        return

    logger.info('Setting execute permission for JADX directory')
    directory_path.chmod(0o755)

    # Recursively set permissions for all files and
    # directories within the root directory
    for path in directory_path.rglob('*'):
        path.chmod(0o755)


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    install_jadx(sys.argv[1])



# -*- coding: utf_8 -*-
"""Dynamic Analyzer Reporting."""
import logging
from pathlib import Path

from django.conf import settings
from django.shortcuts import render
from django.template.defaulttags import register

import mobsf.MalwareAnalyzer.views.Trackers as Trackers
from mobsf.DynamicAnalyzer.views.ios.analysis import (
    get_screenshots,
    ios_api_analysis,
    run_analysis,
)
from mobsf.MobSF.utils import (
    base64_decode,
    common_check,
    get_md5,
    key,
    pretty_json,
    print_n_send_error_response,
    replace,
    strict_package_check,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)


logger = logging.getLogger(__name__)
register.filter('key', key)
register.filter('replace', replace)
register.filter('pretty_json', pretty_json)
register.filter('base64_decode', base64_decode)


@login_required
def ios_view_report(request, bundle_id, api=False):
    """Dynamic Analysis Report Generation."""
    logger.info('iOS Dynamic Analysis Report Generation')
    try:
        if api:
            instance_id = request.POST.get('instance_id')
        else:
            instance_id = request.GET.get('instance_id')
        if instance_id and not common_check(instance_id):
            dev = instance_id
        else:
            dev = ''
        if not strict_package_check(bundle_id):
            # bundle_id is not validated in REST API.
            # Also bundleid is not strictly validated
            # in URL path.
            return print_n_send_error_response(
                request,
                'Invalid iOS Bundle id',
                api)
        checksum = get_md5(bundle_id.encode('utf-8'))
        app_dir = Path(settings.UPLD_DIR) / checksum
        download_dir = settings.DWD_DIR
        tools_dir = settings.TOOLS_DIR
        frida_log = app_dir / 'mobsf_frida_out.txt'
        data_dir = app_dir / 'DYNAMIC_DeviceData'
        if not (frida_log.exists() or data_dir.exists()):
            msg = ('Dynamic Analysis report is not available '
                   'for this app. Perform Dynamic Analysis '
                   'and generate the report.')
            return print_n_send_error_response(request, msg, api)
        api_analysis = ios_api_analysis(app_dir)
        dump_analysis = run_analysis(app_dir, bundle_id, checksum)
        trk = Trackers.Trackers(checksum, app_dir, tools_dir)
        trackers = trk.get_trackers_domains_or_deps(
            dump_analysis['domains'], None)
        screenshots = get_screenshots(checksum, download_dir)
        context = {
            'hash': checksum,
            'version': settings.MOBSF_VER,
            'title': 'iOS Dynamic Analysis Report',
            'instance_id': dev,
            'bundleid': bundle_id,
            'trackers': trackers,
            'screenshots': screenshots,
            'frida_logs': frida_log.exists(),
        }
        context.update(api_analysis)
        context.update(dump_analysis)
        template = 'dynamic_analysis/ios/dynamic_report.html'
        if api:
            return context
        return render(request, template, context)
    except Exception as exp:
        logger.exception('Dynamic Analysis Report Generation')
        err = f'Error Generating Dynamic Analysis Report. {str(exp)}'
        return print_n_send_error_response(request, err, api)



# -*- coding: utf_8 -*-
# flake8: noqa
import os

from defusedxml.minidom import parseString

_public_res = None
# copy the newest sdk/platforms/android-?/data/res/values/public.xml here
if _public_res is None:
    _public_res = {}
    root = os.path.dirname(os.path.realpath(__file__))
    xmlfile = os.path.join(root, "public.xml")
    if os.path.isfile(xmlfile):
        with open(xmlfile, "r") as fp:
            _xml = parseString(fp.read())
            for element in _xml.getElementsByTagName("public"):
                _type = element.getAttribute('type')
                _name = element.getAttribute('name')
                _id = int(element.getAttribute('id'), 16)
                if _type not in _public_res:
                    _public_res[_type] = {}
                _public_res[_type][_name] = _id
    else:
        raise Exception(
            "need to copy the sdk/platforms/android-?/data/res/values/public.xml here"
        )

SYSTEM_RESOURCES = {
    "attributes": {
        "forward": {k: v for k, v in _public_res['attr'].items()},
        "inverse": {v: k for k, v in _public_res['attr'].items()},
    },
    "styles": {
        "forward": {k: v for k, v in _public_res['style'].items()},
        "inverse": {v: k for k, v in _public_res['style'].items()},
    },
}



# -*- coding: utf_8 -*-
"""Binary Analysis - Rule Matcher."""
import re
import logging

from mobsf.StaticAnalyzer.views.ios.rules import (
    ipa_rules,
)
from mobsf.MobSF.utils import (
    append_scan_status,
)

logger = logging.getLogger(__name__)


def get_desc(desc_str, match):
    """Generate formatted detailed description with matches."""
    return desc_str.format(
        b', '.join(sorted(set(match))).decode('utf-8', 'ignore'))


def _add_bfindings(findings, desc, detailed_desc, rule):
    """Add Binary Analysis Findings."""
    findings[desc] = {'detailed_desc': detailed_desc,
                      'severity': rule['severity'],
                      'cvss': rule['cvss'],
                      'cwe': rule['cwe'],
                      'owasp-mobile': rule['owasp-mobile'],
                      'masvs': rule['masvs']}


def binary_rule_matcher(checksum, findings, symbols, classdump):
    """Static Analysis Rule Matcher."""
    try:
        data = classdump + '\n'.join(symbols).encode('utf-8')
        for rule in ipa_rules.IPA_RULES:

            # CASE CHECK
            if rule['input_case'] == 'lower':
                tmp_data = data.lower()
            elif rule['input_case'] == 'upper':
                tmp_data = data.upper()
            elif rule['input_case'] == 'exact':
                tmp_data = data

            # MATCH TYPE
            if rule['type'] == 'Regex':
                matched = re.findall(rule['pattern'], tmp_data)
                if matched:
                    detailed_desc = get_desc(
                        rule['detailed_desc'],
                        matched)
                    _add_bfindings(findings,
                                   rule['description'],
                                   detailed_desc,
                                   rule)

            else:
                pre = 'Binary Rule Error'
                msg = f'{pre}\n{rule}'
                logger.error(msg)
                append_scan_status(checksum, pre, msg)
    except Exception as exp:
        msg = 'Error in Binary Rule Processing'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))



# noqa: E800
"""
Django settings for MobSF project.

MobSF and Django settings
"""

import logging
import os

from mobsf.MobSF.init import (
    first_run,
    get_mobsf_home,
    get_mobsf_version,
    get_secret_from_file_or_env,
    load_source,
)

logger = logging.getLogger(__name__)
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#       MOBSF CONFIGURATION
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
BANNER, VERSION, MOBSF_VER = get_mobsf_version()
USE_HOME = True
# True : All Uploads/Downloads will be stored in user's home directory
# False : All Uploads/Downloads will be stored under MobSF root directory

# MobSF Data Directory
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
MOBSF_HOME = get_mobsf_home(USE_HOME, BASE_DIR)
# Download Directory
DWD_DIR = os.path.join(MOBSF_HOME, 'downloads/')
# Screenshot Directory
SCREEN_DIR = os.path.join(MOBSF_HOME, 'downloads/screen/')
# Upload Directory
UPLD_DIR = os.path.join(MOBSF_HOME, 'uploads/')
# Database Directory
DB_DIR = os.path.join(MOBSF_HOME, 'db.sqlite3')
# Signatures used by modules
SIGNATURE_DIR = os.path.join(MOBSF_HOME, 'signatures/')
# Tools Directory
TOOLS_DIR = os.path.join(BASE_DIR, 'DynamicAnalyzer/tools/')
# Downloaded Tools Directory
DOWNLOADED_TOOLS_DIR = os.path.join(MOBSF_HOME, 'tools/')
# Secret File
SECRET_FILE = os.path.join(MOBSF_HOME, 'secret')

# ==========Load MobSF User Settings==========
try:
    if USE_HOME:
        USER_CONFIG = os.path.join(MOBSF_HOME, 'config.py')
        sett = load_source('user_settings', USER_CONFIG)
        locals().update(  # lgtm [py/modification-of-locals]
            {k: v for k, v in list(sett.__dict__.items())
                if not k.startswith('__')})
        CONFIG_HOME = True
    else:
        CONFIG_HOME = False
except Exception:
    logger.exception('Reading Config')
    CONFIG_HOME = False

# ===MOBSF SECRET GENERATION AND DB MIGRATION====
SECRET_KEY = first_run(SECRET_FILE, BASE_DIR, MOBSF_HOME)

# =============ALLOWED DOWNLOAD EXTENSIONS=====
ALLOWED_EXTENSIONS = {
    '.txt': 'text/plain',
    '.png': 'image/png',
    '.jpg': 'image/jpeg',
    '.svg': 'image/svg+xml',
    '.webp': 'image/webp',
    '.zip': 'application/zip',
    '.tar': 'application/x-tar',
    '.apk': 'application/octet-stream',
    '.apks': 'application/octet-stream',
    '.xapk': 'application/octet-stream',
    '.aab': 'application/octet-stream',
    '.ipa': 'application/octet-stream',
    '.jar': 'application/java-archive',
    '.aar': 'application/octet-stream',
    '.so': 'application/octet-stream',
    '.dylib': 'application/octet-stream',
    '.a': 'application/octet-stream',
    '.pcap': 'application/vnd.tcpdump.pcap',
    '.appx': 'application/vns.ms-appx',
}
# =============ALLOWED MIMETYPES=================
APK_MIME = [
    'application/octet-stream',
    'application/vnd.android.package-archive',
    'application/x-zip-compressed',
    'binary/octet-stream',
    'application/java-archive',
    'application/x-authorware-bin',
]
IPA_MIME = [
    'application/iphone',
    'application/octet-stream',
    'application/x-itunes-ipa',
    'application/x-zip-compressed',
    'application/x-ar',
    'text/vnd.a',
    'binary/octet-stream',
]
ZIP_MIME = [
    'application/zip',
    'application/octet-stream',
    'application/x-zip-compressed',
    'binary/octet-stream',
]
APPX_MIME = [
    'application/octet-stream',
    'application/vns.ms-appx',
    'application/x-zip-compressed',
]
# Supported File Extensions
ANDROID_EXTS = (
    'apk', 'xapk', 'apks', 'zip',
    'aab', 'so', 'jar', 'aar',
)
IOS_EXTS = ('ipa', 'dylib', 'a')
WINDOWS_EXTS = ('appx',)
# REST API only mode
# Set MOBSF_API_ONLY to 1 to enable REST API only mode
# In this mode, web UI related urls are disabled.
API_ONLY = os.getenv('MOBSF_API_ONLY', '0')

# -----External URLS--------------------------
MALWARE_DB_URL = 'https://www.malwaredomainlist.com/mdlcsv.php'
MALTRAIL_DB_URL = ('https://raw.githubusercontent.com/stamparm/aux/'
                   'master/maltrail-malware-domains.txt')
VIRUS_TOTAL_BASE_URL = 'https://www.virustotal.com/vtapi/v2/file/'
EXODUS_URL = 'https://reports.exodus-privacy.eu.org'
APPMONSTA_URL = 'https://api.appmonsta.com/v1/stores/android/details/'
ITUNES_URL = 'https://itunes.apple.com/lookup'
GITHUB_URL = ('https://github.com/MobSF/Mobile-Security-Framework-MobSF/'
              'releases/latest')
FRIDA_SERVER = 'https://api.github.com/repos/frida/frida/releases/tags/'
GOOGLE = 'https://www.google.com'
PLAYSTORE = 'https://play.google.com'
BAIDU = 'https://www.baidu.com/'
APKPURE = 'https://m.apkpure.com/android/{}/download?from=details'
APKTADA = 'https://apktada.com/download-apk/'
APKPLZ = 'https://apkplz.net/download-app/'

# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
# ============DJANGO SETTINGS =================
# !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

# Database
# https://docs.djangoproject.com/en/dev/ref/settings/#databases
if (os.environ.get('POSTGRES_USER')
        and (os.environ.get('POSTGRES_PASSWORD')
             or os.environ.get('POSTGRES_PASSWORD_FILE'))
        and os.environ.get('POSTGRES_HOST')):
    # Postgres support
    default = {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': os.getenv('POSTGRES_DB', 'mobsf'),
        'USER': os.environ['POSTGRES_USER'],
        'PASSWORD': get_secret_from_file_or_env('POSTGRES_PASSWORD'),
        'HOST': os.environ['POSTGRES_HOST'],
        'PORT': int(os.getenv('POSTGRES_PORT', 5432)),
    }
else:
    # Sqlite3 support
    default = {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': DB_DIR,
    }
DATABASES = {
    'default': default,
}
# ===============================================
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'
DEBUG = bool(os.getenv('MOBSF_DEBUG', '0') == '1')
DJANGO_LOG_LEVEL = DEBUG
TEMPLATE_DEBUG = DEBUG
ALLOWED_HOSTS = ['127.0.0.1', 'mobsf', '*']
# Application definition
INSTALLED_APPS = (
    # 'django.contrib.admin',
    'django_q',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'mobsf.StaticAnalyzer',
    'mobsf.DynamicAnalyzer',
    'mobsf.MobSF',
    'mobsf.MalwareAnalyzer',
)
MIDDLEWARE = (
    'django.middleware.security.SecurityMiddleware',
    'mobsf.MobSF.views.api.api_middleware.RestApiAuthMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)
ROOT_URLCONF = 'mobsf.MobSF.urls'
WSGI_APPLICATION = 'mobsf.MobSF.wsgi.application'
LANGUAGE_CODE = 'en-us'
TIME_ZONE = os.getenv('TIME_ZONE', 'UTC')
USE_I18N = True
USE_L10N = True
USE_TZ = True
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'DIRS':
            [
                os.path.join(BASE_DIR, 'templates'),
            ],
        'OPTIONS':
            {
                'debug': TEMPLATE_DEBUG,
                'context_processors': [
                    'django.template.context_processors.debug',
                    'django.template.context_processors.request',
                    'django.contrib.auth.context_processors.auth',
                    'django.contrib.messages.context_processors.messages',
                ],
            },
    },
]
MEDIA_ROOT = os.path.join(BASE_DIR, 'uploads')
MEDIA_URL = '/uploads/'
STATIC_URL = '/static/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATICFILES_STORAGE = 'whitenoise.storage.CompressedStaticFilesStorage'
# 256MB limit for file uploads
DATA_UPLOAD_MAX_MEMORY_SIZE = 256 * 1024 * 1024
# 400MB per file limit for uncompressed files
ZIP_MAX_UNCOMPRESSED_FILE_SIZE = 400 * 1024 * 1024
# 3GB total limit for all uncompressed files
ZIP_MAX_UNCOMPRESSED_TOTAL_SIZE = 3000 * 1024 * 1024
LOGIN_URL = 'login'
LOGOUT_REDIRECT_URL = '/'
AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': ('django.contrib.auth.password_validation.'
                 'UserAttributeSimilarityValidator'),
    },
    {
        'NAME': ('django.contrib.auth.password_validation.'
                 'MinimumLengthValidator'),
        'OPTIONS': {
            'min_length': 6,
        },
    },
    {
        'NAME': ('django.contrib.auth.password_validation.'
                 'CommonPasswordValidator'),
    },
    {
        'NAME': ('django.contrib.auth.password_validation.'
                 'NumericPasswordValidator'),
    },
]
# Better logging
LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'standard': {
            'format': '[%(levelname)s] %(asctime)-15s - %(message)s',
            'datefmt': '%d/%b/%Y %H:%M:%S',
        },
        'color': {
            '()': 'colorlog.ColoredFormatter',
            'format':
                '%(log_color)s[%(levelname)s] %(asctime)-15s - %(message)s',
            'datefmt': '%d/%b/%Y %H:%M:%S',
            'log_colors': {
                'DEBUG': 'cyan',
                'INFO': 'green',
                'WARNING': 'yellow',
                'ERROR': 'red',
                'CRITICAL': 'red,bg_white',
            },
        },
    },
    'handlers': {
        'logfile': {
            'level': 'DEBUG',
            'class': 'logging.FileHandler',
            'filename': os.path.join(MOBSF_HOME, 'debug.log'),
            'formatter': 'standard',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'color',
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console', 'logfile'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'django_q': {
            'handlers': ['console', 'logfile'],
            'level': 'DEBUG',
            'propagate': True,
        },
        'django.db.backends': {
            'handlers': ['console', 'logfile'],
            # DEBUG will log all queries, so change it to WARNING.
            'level': 'INFO',
            'propagate': False,   # Don't propagate to other handlers
        },
        'mobsf.MobSF': {
            'handlers': ['console', 'logfile'],
            'level': 'DEBUG',
            'propagate': False,
        },
        'mobsf.StaticAnalyzer': {
            'handlers': ['console', 'logfile'],
            'level': 'DEBUG',
            'propagate': False,
        },
        'mobsf.MalwareAnalyzer': {
            'handlers': ['console', 'logfile'],
            'level': 'DEBUG',
            'propagate': False,
        },
        'mobsf.DynamicAnalyzer': {
            'handlers': ['console', 'logfile'],
            'level': 'DEBUG',
            'propagate': False,
        },
    },
}
ASYNC_ANALYSIS = bool(os.getenv('MOBSF_ASYNC_ANALYSIS', '0') == '1')
ASYNC_ANALYSIS_TIMEOUT = int(os.getenv('MOBSF_ASYNC_ANALYSIS_TIMEOUT', '60'))
Q_CLUSTER = {
    'name': 'scan_queue',
    'workers': int(os.getenv('MOBSF_ASYNC_WORKERS', '2')),
    'recycle': 100,
    'timeout': ASYNC_ANALYSIS_TIMEOUT * 60,
    'retry': (ASYNC_ANALYSIS_TIMEOUT * 60) + 100,
    'compress': True,
    'label': 'scan_queue',
    'orm': 'default',
    'max_attempts': 1,
    'save_limit': -1,
    'ack_failures': True,
}
QUEUE_MAX_SIZE = 100
MULTIPROCESSING = os.getenv('MOBSF_MULTIPROCESSING')
JADX_TIMEOUT = int(os.getenv('MOBSF_JADX_TIMEOUT', 1000))
SAST_TIMEOUT = int(os.getenv('MOBSF_SAST_TIMEOUT', 1000))
BINARY_ANALYSIS_TIMEOUT = int(os.getenv('MOBSF_BINARY_ANALYSIS_TIMEOUT', 600))
DISABLE_AUTHENTICATION = os.getenv('MOBSF_DISABLE_AUTHENTICATION')
RATELIMIT = os.getenv('MOBSF_RATELIMIT', '7/m')
USE_X_FORWARDED_HOST = bool(
    os.getenv('MOBSF_USE_X_FORWARDED_HOST', '1') == '1')
USE_X_FORWARDED_PORT = bool(
    os.getenv('MOBSF_USE_X_FORWARDED_PORT', '1') == '1')
SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
# ===========================
# ENTERPRISE FEATURE REQUESTS
# ===========================
EFR_01 = os.getenv('EFR_01', '0')
# SAML SSO
# IdP Configuration
IDP_METADATA_URL = os.getenv('MOBSF_IDP_METADATA_URL')
IDP_ENTITY_ID = os.getenv('MOBSF_IDP_ENTITY_ID')
IDP_SSO_URL = os.getenv('MOBSF_IDP_SSO_URL')
IDP_X509CERT = os.getenv('MOBSF_IDP_X509CERT')
IDP_IS_ADFS = os.getenv('MOBSF_IDP_IS_ADFS', '0')
IDP_MAINTAINER_GROUP = os.getenv('MOBSF_IDP_MAINTAINER_GROUP', 'Maintainer')
IDP_VIEWER_GROUP = os.getenv('MOBSF_IDP_VIEWER_GROUP', 'Viewer')
# SP Configuration
SP_HOST = os.getenv('MOBSF_SP_HOST')
SP_ALLOW_PASSWORD = os.getenv('MOBSF_SP_ALLOW_PASSWORD', '0')
# ===================
# USER CONFIGURATION
# ===================
if CONFIG_HOME:
    logger.info('Loading User config from: %s', USER_CONFIG)
else:
    """
    IMPORTANT
    If 'USE_HOME' is set to True,
    then below user configuration settings are not considered.
    The user configuration will be loaded from
    .MobSF/config.py in user's home directory.
    """
    # ^CONFIG-START^: Do not edit this line
    # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    #          MOBSF USER CONFIGURATIONS
    # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    # -------------------------
    # STATIC ANALYZER SETTINGS
    # -------------------------

    # ==========ANDROID SKIP CLASSES==========================
    # Common third party classes/paths that will be skipped
    # during static analysis
    import os
    SKIP_CLASS_PATH = {
        'com/google/', 'androidx', 'okhttp2/', 'okhttp3/',
        'com/android/', 'com/squareup', 'okhttp/'
        'android/content/', 'com/twitter/', 'twitter4j/',
        'android/support/', 'org/apache/', 'oauth/signpost',
        'android/arch', 'org/chromium/', 'com/facebook',
        'org/spongycastle', 'org/bouncycastle',
        'com/amazon/identity/', 'io/fabric/sdk',
        'com/instabug', 'com/crashlytics/android',
        'kotlinx/', 'kotlin/',
    }
    # Disable CVSSV2 Score by default
    CVSS_SCORE_ENABLED = bool(os.getenv('MOBSF_CVSS_SCORE_ENABLED', ''))
    # NIAP Scan
    NIAP_ENABLED = os.getenv('MOBSF_NIAP_ENABLED', '')
    # Permission to Code Mapping
    PERM_MAPPING_ENABLED = os.getenv('MOBSF_PERM_MAPPING_ENABLED', '1')
    # Dex 2 Smali Conversion
    DEX2SMALI_ENABLED = os.getenv('MOBSF_DEX2SMALI_ENABLED', '1')
    # Android Shared Object Binary Analysis
    SO_ANALYSIS_ENABLED = os.getenv('MOBSF_SO_ANALYSIS_ENABLED', '1')
    # iOS Dynamic Library Binary Analysis
    DYLIB_ANALYSIS_ENABLED = os.getenv('MOBSF_DYLIB_ANALYSIS_ENABLED', '1')
    # =================================================
    # --------------------------
    # MALWARE ANALYZER SETTINGS
    # --------------------------

    DOMAIN_MALWARE_SCAN = os.getenv('MOBSF_DOMAIN_MALWARE_SCAN', '1')
    APKID_ENABLED = os.getenv('MOBSF_APKID_ENABLED', '1')
    # ==================================================
    # ======WINDOWS STATIC ANALYSIS SETTINGS ===========
    # Private key
    WINDOWS_VM_SECRET = os.getenv(
        'MOBSF_WINDOWS_VM_SECRET', 'mobsf/MobSF/windows_vm_priv_key.asc')
    # IP and Port of the MobSF Windows VM
    # example: WINDOWS_VM_IP = '127.0.0.1'   ;noqa E800
    WINDOWS_VM_IP = os.getenv('MOBSF_WINDOWS_VM_IP')
    WINDOWS_VM_PORT = os.getenv('MOBSF_WINDOWS_VM_PORT', '8000')
    # ==================================================

    # ==============3rd Party Tools=====================
    """
    If you want to use a different version of 3rd party tools used by MobSF.
    You can do that by specifying the path here. If specified, MobSF will run
    the tool from this location.
    """

    # Android 3P Tools
    BUNDLE_TOOL = os.getenv('MOBSF_BUNDLE_TOOL', '')
    JADX_BINARY = os.getenv('MOBSF_JADX_BINARY', '')
    BACKSMALI_BINARY = os.getenv('MOBSF_BACKSMALI_BINARY', '')
    VD2SVG_BINARY = os.getenv('MOBSF_VD2SVG_BINARY', '')
    APKTOOL_BINARY = os.getenv('MOBSF_APKTOOL_BINARY', '')
    ADB_BINARY = os.getenv('MOBSF_ADB_BINARY', '')
    AAPT2_BINARY = os.getenv('MOBSF_AAPT2_BINARY', '')
    AAPT_BINARY = os.getenv('MOBSF_AAPT_BINARY', '')

    # iOS 3P Tools
    JTOOL_BINARY = os.getenv('MOBSF_JTOOL_BINARY', '')
    CLASSDUMP_BINARY = os.getenv('MOBSF_CLASSDUMP_BINARY', '')
    CLASSDUMP_SWIFT_BINARY = os.getenv('MOBSF_CLASSDUMP_SWIFT_BINARY', '')

    # COMMON
    JAVA_DIRECTORY = os.getenv('MOBSF_JAVA_DIRECTORY', '')

    """
    Examples:
    JAVA_DIRECTORY = 'C:/Program Files/Java/jdk1.7.0_17/bin/'
    JAVA_DIRECTORY = '/usr/bin/'
    JADX_BINARY = 'C:/Users/Ajin/AppData/Local/Programs/jadx/bin/jadx.bat'
    JADX_BINARY = '/Users/ajin/jadx/bin/jadx'
    """
    # ==========================================================
    # -------------------------
    # DYNAMIC ANALYZER SETTINGS
    # -------------------------

    # =======ANDROID DYNAMIC ANALYSIS SETTINGS===========
    ANALYZER_IDENTIFIER = os.getenv('MOBSF_ANALYZER_IDENTIFIER', '')
    FRIDA_TIMEOUT = int(os.getenv('MOBSF_FRIDA_TIMEOUT', '4'))
    ACTIVITY_TESTER_SLEEP = int(os.getenv('MOBSF_ACTIVITY_TESTER_SLEEP', '4'))
    # ==============================================

    # ================HTTPS PROXY ===============
    PROXY_IP = os.getenv('MOBSF_PROXY_IP', '127.0.0.1')
    PROXY_PORT = int(os.getenv('MOBSF_PROXY_PORT', '1337'))
    # ===================================================

    # ========UPSTREAM PROXY SETTINGS ==============
    # If you are behind a Proxy
    UPSTREAM_PROXY_ENABLED = bool(os.getenv(
        'MOBSF_UPSTREAM_PROXY_ENABLED', ''))
    UPSTREAM_PROXY_SSL_VERIFY = os.getenv(
        'MOBSF_UPSTREAM_PROXY_SSL_VERIFY', '1')
    UPSTREAM_PROXY_TYPE = os.getenv('MOBSF_UPSTREAM_PROXY_TYPE', 'http')
    UPSTREAM_PROXY_IP = os.getenv('MOBSF_UPSTREAM_PROXY_IP', '127.0.0.1')
    UPSTREAM_PROXY_PORT = int(os.getenv('MOBSF_UPSTREAM_PROXY_PORT', '3128'))
    UPSTREAM_PROXY_USERNAME = os.getenv('MOBSF_UPSTREAM_PROXY_USERNAME', '')
    UPSTREAM_PROXY_PASSWORD = os.getenv('MOBSF_UPSTREAM_PROXY_PASSWORD', '')
    # ==============================================

    # ========DISABLED BY DEFAULT COMPONENTS=========
    # Get AppMonsta API from https://appmonsta.com/dashboard/get_api_key/
    APPMONSTA_API = os.getenv('MOBSF_APPMONSTA_API', '')
    # ----------VirusTotal--------------------------
    VT_ENABLED = bool(os.getenv('MOBSF_VT_ENABLED', ''))
    VT_API_KEY = os.getenv('MOBSF_VT_API_KEY', '')
    VT_UPLOAD = bool(os.getenv('MOBSF_VT_UPLOAD', ''))
    # Before setting VT_ENABLED to True,
    # Make sure VT_API_KEY is set to your VirusTotal API key
    # register at: https://www.virustotal.com/#/join-us
    # You can get your API KEY from:
    # https://www.virustotal.com/en/user/<username>/apikey/
    # Files will be uploaded to VirusTotal
    # if VT_UPLOAD is set to True.
    # ===============================================
    # =======IOS DYNAMIC ANALYSIS SETTINGS===========
    # Should be SSH IP:PORT, example: 192.168.1.100:22
    # Field also supports multiple devices, example: 192.168.1.100:22,192.168.1.101:22
    IOS_ANALYZER_IDENTIFIERS = os.getenv('MOBSF_IOS_ANALYZER_IDENTIFIERS', '')
    # SSH credentials for jailbroken iOS device (USB or WiFi path)
    # Defaults match the standard jailbreak SSH default (root/alpine)
    IOS_SSH_USER = os.getenv('MOBSF_IOS_SSH_USER', 'root')
    IOS_SSH_PASSWORD = os.getenv('MOBSF_IOS_SSH_PASSWORD', 'alpine')
    # ==============================================

    # =======IOS DYNAMIC ANALYSIS CORELLIUM SETTINGS===========
    CORELLIUM_API_DOMAIN = os.getenv('MOBSF_CORELLIUM_API_DOMAIN', '')
    CORELLIUM_API_KEY = os.getenv('MOBSF_CORELLIUM_API_KEY', '')
    CORELLIUM_PROJECT_ID = os.getenv('MOBSF_CORELLIUM_PROJECT_ID', '')
    # CORELLIUM_PROJECT_ID is optional, MobSF will use any available project id
    # ===============================================
    # ^CONFIG-END^: Do not edit this line



from pathlib import Path

from django.conf import settings

from mobsf.MobSF.utils import strict_package_check


def get_content(file_name):
    tools_dir = Path(settings.TOOLS_DIR)
    aux_dir = tools_dir / 'frida_scripts' / 'android' / 'auxiliary'
    script = aux_dir / file_name

    if script.exists():
        return script.read_text('utf-8', 'ignore')
    return ''


def get_loaded_classes():
    """Get Loaded classes."""
    return get_content('get_loaded_classes.js')


def string_catch():
    """Capture all runtime strings."""
    return get_content('string_catch.js')


def string_compare():
    """Capture all runtime string comparisons."""
    return get_content('string_compare.js')


def get_methods(klazz):
    """Get Class methods and implementations."""
    if not strict_package_check(klazz):
        return ''
    content = get_content('get_methods.js')
    return content.replace('{{CLASS}}', klazz)


def class_pattern(pattern):
    """Search in loaded classes based on pattern."""
    pattern = pattern.replace(
        '/', '\\/').replace(';', '').replace('\n', '').replace('\r', '')
    content = get_content('search_class_pattern.js')
    return content.replace('{{PATTERN}}', pattern)


def class_trace(classes):
    """Trace all methods of a class."""
    filtered = []
    if ',' not in classes:
        filtered.append(classes.strip())
    else:
        for clz in classes.split(','):
            filtered.append(clz.strip())
    for klz in filtered:
        if not strict_package_check(klz):
            return ''
    content = get_content('class_trace.js')
    return content.replace('{{CLASSES}}', str(filtered))



# -*- coding: utf_8 -*-
"""Frida tests."""
import base64
import os
import re
import json
from pathlib import Path
from threading import Thread
import logging

from django.shortcuts import render
from django.conf import settings
from django.views.decorators.http import require_http_methods

from mobsf.DynamicAnalyzer.views.android.frida_core import Frida
from mobsf.DynamicAnalyzer.views.common.shared import (
    invalid_params,
    send_response,
)
from mobsf.MobSF.security import is_attack_pattern
from mobsf.DynamicAnalyzer.views.android.operations import (
    get_package_name,
)
from mobsf.MobSF.utils import (
    is_file_exists,
    is_md5,
    print_n_send_error_response,
    strict_package_check,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)

logger = logging.getLogger(__name__)

# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def get_runtime_dependencies(request, api=False):
    """Get App runtime dependencies."""
    data = {
        'status': 'failed',
        'message': 'Failed to get runtime dependencies'}
    try:
        checksum = request.POST['hash']
        if not is_md5(checksum):
            return invalid_params(api)
        package = get_package_name(checksum)
        if not package:
            return invalid_params(api)
        get_dependencies(package, checksum)
        return send_response(
            {'status': 'ok'},
            api)
    except Exception:
        pass
    return send_response(data, api)
# AJAX


@login_required
@permission_required(Permissions.SCAN)
@require_http_methods(['POST'])
def instrument(request, api=False):
    """Instrument app with frida."""
    data = {
        'status': 'failed',
        'message': ''}
    try:
        action = request.POST.get('frida_action', 'spawn')
        pid = request.POST.get('pid')
        new_pkg = request.POST.get('new_package')
        md5_hash = request.POST['hash']
        default_hooks = request.POST['default_hooks']
        auxiliary_hooks = request.POST['auxiliary_hooks']
        code = request.POST['frida_code']
        # Fill extras
        extras = {}
        class_name = request.POST.get('class_name')
        if class_name:
            extras['class_name'] = class_name.strip()
        class_search = request.POST.get('class_search')
        if class_search:
            extras['class_search'] = class_search.strip()
        cls_trace = request.POST.get('class_trace')
        if cls_trace:
            extras['class_trace'] = cls_trace.strip()

        if (is_attack_pattern(default_hooks)
                or is_attack_pattern(auxiliary_hooks)
                or not is_md5(md5_hash)
                or (new_pkg and not strict_package_check(new_pkg))):
            return invalid_params(api)
        package = get_package_name(md5_hash)
        if not package and not new_pkg:
            return invalid_params(api)
        frida_obj = Frida(md5_hash,
                          package,
                          default_hooks.split(','),
                          auxiliary_hooks.split(','),
                          extras,
                          code)
        if action == 'spawn':
            logger.info('Starting Instrumentation')
            frida_obj.spawn()
        elif action == 'ps':
            logger.info('Enumerating running applications')
            data['message'] = frida_obj.ps()
        elif action == 'get':
            # Get injected Frida script.
            data['message'] = frida_obj.get_script(nolog=True)
        if action in ('spawn', 'session'):
            if pid and pid.isdigit():
                # Attach to a different pid/bundle id
                args = (int(pid), new_pkg)
                logger.info('Attaching to %s [PID: %s]', new_pkg, pid)
            else:
                # Injecting to existing session/spawn
                if action == 'session':
                    logger.info('Injecting to existing frida session')
                args = (None, None)
            Thread(target=frida_obj.session, args=args, daemon=True).start()
        data['status'] = 'ok'
    except Exception as exp:
        logger.exception('Instrumentation failed')
        data = {'status': 'failed', 'message': str(exp)}
    return send_response(data, api)


@login_required
def live_api(request, api=False):
    try:
        if api:
            apphash = request.POST['hash']
            stream = True
        else:
            apphash = request.GET.get('hash', '')
            stream = request.GET.get('stream', '')
        if not is_md5(apphash):
            return invalid_params(api)
        if stream:
            apk_dir = os.path.join(settings.UPLD_DIR, apphash + '/')
            apimon_file = os.path.join(apk_dir, 'mobsf_api_monitor.txt')
            data = {}
            if not is_file_exists(apimon_file):
                data = {
                    'status': 'failed',
                    'message': 'Data does not exist.'}
                return send_response(data, api)
            with open(apimon_file, 'r',
                      encoding='utf8',
                      errors='ignore') as flip:
                api_list = json.loads('[{}]'.format(
                    flip.read()[:-1]))
            data = {'data': api_list}
            return send_response(data, api)
        logger.info('Starting API monitor streaming')
        template = 'dynamic_analysis/android/live_api.html'
        return render(request,
                      template,
                      {'hash': apphash,
                       'package': request.GET.get('package', ''),
                       'version': settings.MOBSF_VER,
                       'title': 'Live API Monitor'})
    except Exception:
        logger.exception('API monitor streaming')
        err = 'Error in API monitor streaming'
        return print_n_send_error_response(request, err, api)


def decode_base64(data, altchars=b'+/'):
    """Decode base64, padding being optional.

    :param data: Base64 data as an ASCII byte string
    :returns: The decoded byte string.

    """
    data = re.sub(rb'[^a-zA-Z0-9%s]+' % altchars,
                  b'', data.encode())
    missing_padding = len(data) % 4
    if missing_padding:
        data += b'=' * (4 - missing_padding)
    return base64.b64decode(data, altchars)


def get_icon_map(name):
    """Get icon mapping."""
    mapping = {
        'Process': 'fas fa-chart-bar',
        'Command': 'fas fa-terminal',
        'Java Native Interface': 'fab fa-cuttlefish',
        'WebView': 'far fa-window-maximize',
        'File IO': 'fas fa-file-signature',
        'Database': 'fas fa-database',
        'IPC': 'fas fa-broadcast-tower',
        'Binder': 'fas fa-cubes',
        'Crypto': 'fas fa-lock',
        'Crypto - Hash': 'fas fa-hashtag',
        'Device Info': 'fas fa-info',
        'Network': 'fas fa-wifi',
        'Dex Class Loader': 'fas fa-asterisk',
        'Base64': 'fas fa-puzzle-piece',
        'System Manager': 'fas fa-cogs',
        'SMS': 'fas fa-comment-alt',
        'Device Data': 'fas fa-phone',
    }
    if name in mapping:
        return mapping[name]
    return 'far fa-dot-circle'


def apimon_analysis(app_dir):
    """API Analysis."""
    api_details = {}
    try:
        strings = []
        location = os.path.join(app_dir, 'mobsf_api_monitor.txt')
        if not is_file_exists(location):
            return api_details, strings
        logger.info('Frida API Monitor Analysis')
        with open(location, 'r',
                  encoding='utf8',
                  errors='ignore') as flip:
            apis = json.loads('[{}]'.format(
                flip.read()[:-1]))
        for api in apis:
            to_decode = None
            if (api['class'] == 'android.util.Base64'
                    and (api['method'] == 'encodeToString')):
                if api.get('returnValue'):
                    to_decode = api['returnValue'].replace('"', '')
            elif (api['class'] == 'android.util.Base64'
                  and api['method'] == 'decode'):
                to_decode = api['arguments'][0]
            try:
                if to_decode:
                    api['decoded'] = decode_base64(
                        to_decode).decode('utf-8', 'ignore')
                    strings.append((api['calledFrom'], api['decoded']))
            except Exception:
                pass
            api['icon'] = get_icon_map(api['name'])
            if api['name'] in api_details:
                api_details[api['name']].append(api)
            else:
                api_details[api['name']] = [api]
    except Exception:
        logger.exception('API Monitor Analysis')
    return api_details, strings


def get_dependencies(package, checksum):
    """Get 3rd party dependencies at runtime."""
    frd = Frida(
        checksum,
        package,
        ['ssl_pinning_bypass', 'debugger_check_bypass', 'root_bypass'],
        ['get_dependencies'],
        None,
        None,
    )
    location = Path(frd.deps)
    if location.exists():
        location.write_text('')
    frd.spawn()
    Thread(target=frd.session, args=(None, None), daemon=True).start()


def dependency_analysis(package, app_dir):
    deps = set()
    msg = 'Collecting Runtime Dependency Analysis data'
    try:
        ignore = (
            package,
            'android.', 'androidx.', 'kotlin.', 'kotlinx.', 'java.', 'javax.',
            'sun.', 'com.android.', 'j$', 'dalvik.system.', 'libcore.',
            'com.google.', 'org.kxml2.', 'org.apache.', 'org.json.')
        location = Path(app_dir) / 'mobsf_app_deps.txt'
        if not location.exists():
            return deps
        logger.info(msg)
        data = location.read_text('utf-8', 'ignore').splitlines()
        for dep in data:
            if not dep.startswith(ignore):
                deps.add(dep.rsplit('.', 1)[0])
    except Exception:
        logger.exception(msg)
    return deps



"""Runtime Executable Tampering Detection."""
import subprocess
import functools
import logging
import re
import socket
import ipaddress
import sys
from shutil import which
from pathlib import Path
from platform import system
import os
import stat
import string
import unicodedata
from urllib.parse import unquote, urlparse
from concurrent.futures import ThreadPoolExecutor

from mobsf.MobSF.utils import (
    find_aapt,
    find_java_binary,
    gen_sha256_hash,
    get_adb,
    sha256,
)

from django.conf import settings


logger = logging.getLogger(__name__)
# Non executable files at host level
_SKIP = [
    '.pyc', '.js',
    '.json', '.txt', '.md']
EXECUTABLE_HASH_MAP = None


def get_sha256(filepath):
    """Calculate sha256 hash of a file."""
    return (filepath.as_posix(), sha256(filepath))


def get_all_files(dirlocs):
    """Get all files from a list of directories/files."""
    for dirloc in dirlocs:
        if dirloc.is_file() and dirloc.suffix not in _SKIP:
            yield dirloc
        elif dirloc.is_dir():
            # Use a generator expression for efficient filtering
            files_in_dir = (
                efile for efile in dirloc.rglob('*')
                if efile.is_file() and efile.suffix not in _SKIP
            )
            # Yield all files from the filtered generator
            yield from files_in_dir


def generate_hashes(dirlocs):
    """Generate master hash for all files."""
    exec_hashes = {}
    with ThreadPoolExecutor() as executor:
        futures = []
        for efile in get_all_files(dirlocs):
            futures.append(
                executor.submit(get_sha256, efile))
        for future in futures:
            sha = future.result()
            exec_hashes[sha[0]] = sha[1]
    return exec_hashes, gen_sha256_hash(str(exec_hashes))


def get_executable_hashes():
    # Internal Binaries shipped with MobSF
    base = Path(settings.BASE_DIR)
    downloaded_tools = Path(settings.DOWNLOADED_TOOLS_DIR)
    manage_py = base.parent / 'manage.py'
    exec_loc = [
        base / 'DynamicAnalyzer' / 'tools',
        base / 'StaticAnalyzer' / 'tools',
        downloaded_tools,
        manage_py,
    ]
    aapt = 'aapt'
    aapt2 = 'aapt2'
    if system() == 'Windows':
        aapt = 'aapt.exe'
        aapt2 = 'aapt2.exe'
    aapts = [find_aapt(aapt), find_aapt(aapt2)]
    exec_loc.extend(Path(a) for a in aapts if a)
    # External binaries used directly by MobSF
    system_bins = [
        'aapt',
        'aapt.exe',
        'aapt2',
        'aapt2.exe',
        'adb',
        'adb.exe',
        'which',
        'wkhtmltopdf',
        'httptools',
        'mitmdump',
        'unzip',
        'lipo',
        'ar',
        'nm',
        'objdump',
        'strings',
        'xcrun',
        'BinSkim.exe',
        'BinScope.exe',
        'nuget.exe',
        'where.exe',
        'wkhtmltopdf.exe',
        'idevice_id',
        'ideviceinfo',
        'idevicename',
        'pkill',
        'iproxy',
    ]
    for sbin in system_bins:
        bin_path = which(sbin)
        if bin_path:
            exec_loc.append(Path(bin_path))
    # User defined path/binaries
    if settings.JAVA_DIRECTORY:
        exec_loc.append(Path(settings.JAVA_DIRECTORY))
    user_defined_bins = [
        sys.executable,
        settings.JADX_BINARY,
        settings.BACKSMALI_BINARY,
        settings.VD2SVG_BINARY,
        settings.APKTOOL_BINARY,
        settings.ADB_BINARY,
        settings.JTOOL_BINARY,
        settings.CLASSDUMP_BINARY,
        settings.CLASSDUMP_SWIFT_BINARY,
        getattr(settings, 'BUNDLE_TOOL', ''),
        getattr(settings, 'AAPT2_BINARY', ''),
        getattr(settings, 'AAPT_BINARY', ''),
    ]
    for ubin in user_defined_bins:
        if ubin:
            exec_loc.append(Path(ubin))
    # Add ADB and Java binaries
    adb = get_adb()
    java = find_java_binary()
    if adb == 'adb':
        adb = which('adb')
    if java == 'java':
        java = which('java')
    if adb:
        exec_loc.append(Path(adb))
    if java:
        exec_loc.append(Path(java))
    return generate_hashes(exec_loc)


def store_exec_hashes_at_first_run():
    """Store executable hashes at first run."""
    global EXECUTABLE_HASH_MAP
    try:
        hashes, signature = get_executable_hashes()
        hashes['signature'] = signature
        EXECUTABLE_HASH_MAP = hashes
    except Exception:
        logger.exception('Cannot calculate executable hashes, '
                         'disabling runtime executable '
                         'tampering detection')


def subprocess_hook(oldfunc, *args, **kwargs):
    if isinstance(args[0], str):
        # arg is a string
        agmtz = args[0].split()
        exec1 = agmtz[0]
    else:
        # list of args
        agmtz = args[0]
        exec1 = agmtz[0]  # executable
    exec2 = None  # secondary executable
    for arg in agmtz:
        if arg.endswith('.jar'):
            exec2 = Path(arg).as_posix()
            break
    if '/' in exec1 or '\\' in exec1:
        exec1 = Path(exec1).as_posix()
    else:
        exec1 = Path(which(exec1)).as_posix()
    executable_in_hash_map = False
    if exec1 in EXECUTABLE_HASH_MAP:
        executable_in_hash_map = True
        if EXECUTABLE_HASH_MAP[exec1] != sha256(exec1):
            msg = (
                f'Executable Tampering Detected. [{exec1}]'
                ' has been modified during runtime')
            logger.error(msg)
            raise Exception(msg)
    if exec2 and exec2 in EXECUTABLE_HASH_MAP:
        executable_in_hash_map = True
        if EXECUTABLE_HASH_MAP[exec2] != sha256(exec2):
            msg = (
                f'JAR Tampering Detected. [{exec2}]'
                ' has been modified during runtime')
            logger.error(msg)
            raise Exception(msg)
    if not executable_in_hash_map:
        logger.warning('Executable [%s] not found in known hashes, '
                       'skipping runtime executable '
                       'tampering detection', exec1)
        _, signature = get_executable_hashes()
        if EXECUTABLE_HASH_MAP['signature'] != signature:
            msg = 'Executable/Library Tampering Detected'
            logger.error(msg)
            raise Exception(msg)
    return oldfunc(*args, **kwargs)


def init_exec_hooks():
    subprocess.Popen = wrap_function(
        subprocess.Popen,
        subprocess_hook)


def wrap_function(oldfunction, newfunction):
    @functools.wraps(oldfunction)
    def run(*args, **kwargs):
        return newfunction(oldfunction, *args, **kwargs)
    return run


def sanitize_redirect(url):
    """Sanitize Redirect URL."""
    root = '/'
    if url.startswith('//'):
        return root
    elif url.startswith('/'):
        return url
    return root


def sanitize_filename(filename):
    """Sanitize Filename."""
    # Remove any characters
    # that are not alphanumeric, hyphens, underscores, or dots
    safe_filename = re.sub(r'[^a-zA-Z0-9._-]', '_', filename)
    # Merge multiple underscores into one
    safe_filename = re.sub(r'__+', '_', safe_filename)
    # Remove leading and trailing underscores
    safe_filename = safe_filename.strip('_')
    return safe_filename


def sanitize_for_logging(filename: str, max_length: int = 255) -> str:
    """Sanitize a filename to prevent log injection."""
    # Remove newline, carriage return, and other risky characters
    filename = filename.replace('\n', '_').replace('\r', '_').replace('\t', '_')

    # Allow only safe characters (alphanumeric, underscore, dash, and period)
    filename = re.sub(r'[^a-zA-Z0-9._-]', '_', filename)

    # Truncate filename to the maximum allowed length
    return filename[:max_length]


def valid_host(host):
    """Check if host is valid, run SSRF checks."""
    default_timeout = socket.getdefaulttimeout()
    try:
        if len(host) > 2083:  # Standard URL length limit
            return False

        prefixs = ('http://', 'https://')
        if not host.startswith(prefixs):
            host = f'http://{host}'
        parsed = urlparse(host)
        scheme = parsed.scheme
        domain = parsed.netloc
        hostname = parsed.hostname
        path = parsed.path
        query = parsed.query
        params = parsed.params
        port = parsed.port

        # Allow only http and https schemes
        if scheme not in ('http', 'https'):
            return False

        # Check for hostname
        if not hostname:
            return False

        # Validate port - only allow 80 and 443
        if port and port not in (80, 443):
            return False

        # Check for URL credentials
        if '@' in domain:
            return False

        # Detect parser escapes, only host is allowed
        if len(path) > 0 or len(query) > 0 or len(params) > 0:
            return False

        # Resolve dns to get ipv4 or ipv6 address
        socket.setdefaulttimeout(5)  # 5 second timeout
        ip_addresses = socket.getaddrinfo(hostname, None)
        for ip in ip_addresses:
            ip_obj = ipaddress.ip_address(ip[4][0])
            if (ip_obj.is_private
                or ip_obj.is_loopback
                or ip_obj.is_link_local
                or ip_obj.is_reserved
                or ip_obj.is_multicast
                    or ip_obj.is_unspecified):
                return False

            # Additional checks for specific IPv4 ranges
            if isinstance(ip_obj, ipaddress.IPv4Address):
                problematic_networks = [
                    '127.0.0.0/8',
                    '169.254.0.0/16',
                    '172.16.0.0/12',
                    '192.168.0.0/16',
                    '10.0.0.0/8',
                ]
                for network in problematic_networks:
                    if ip_obj in ipaddress.IPv4Network(network):
                        return False

        # If all checks pass, return True
        return True
    except Exception:
        return False
    finally:
        # Restore default socket timeout
        socket.setdefaulttimeout(default_timeout)


def sanitize_svg(svg_content):
    """Sanitize SVG content to prevent XSS attacks."""
    logger.info('Sanitizing SVG contents')
    import bleach
    # Allow standard SVG tags and attributes, but remove scripts and event handlers
    safe_tags = [
        'svg', 'g', 'path', 'rect', 'circle', 'ellipse',
        'line', 'polyline', 'polygon', 'text', 'tspan',
        'defs', 'use', 'image', 'mask', 'clipPath',
        'filter', 'linearGradient', 'radialGradient', 'stop',
    ]
    safe_attrs = {
        '*': ['id', 'class', 'transform', 'fill', 'stroke', 'stroke-width', 'opacity'],
        'svg': ['width', 'height', 'viewBox', 'xmlns', 'version'],
        'path': ['d'],
        'rect': ['x', 'y', 'width', 'height', 'rx', 'ry'],
        'circle': ['cx', 'cy', 'r'],
        'ellipse': ['cx', 'cy', 'rx', 'ry'],
        'line': ['x1', 'y1', 'x2', 'y2'],
        'polyline': ['points'],
        'polygon': ['points'],
        'text': ['x', 'y', 'font-family', 'font-size'],
        'image': ['x', 'y', 'width', 'height', 'href'],
        'use': ['x', 'y', 'width', 'height', 'href'],
        'filter': ['x', 'y', 'width', 'height', 'href'],
        'linearGradient': ['x1', 'y1', 'x2', 'y2'],
        'radialGradient': ['cx', 'cy', 'r', 'fx', 'fy'],
        'stop': ['offset', 'stop-color', 'stop-opacity'],
    }
    return bleach.clean(
        svg_content,
        tags=safe_tags,
        attributes=safe_attrs,
        strip=True,
    )


def is_path_traversal(user_input):
    """Check for path traversal."""
    if not user_input:
        return False

    # Disallow absolute paths and windows paths and backslashes
    if os.path.isabs(user_input) or user_input.startswith(('\\', '//')):
        logger.error('Path traversal attack detected with absolute path')
        return True

    # Normalize and decode URL-encoded characters
    try:
        # Handle URL decoding (e.g., %2e -> .)
        decoded = unquote(user_input)
        # Handle double URL decoding (e.g., %252e -> %2e -> .)
        double_decoded = unquote(decoded)
    except Exception:
        logger.error('Path traversal attack detected with invalid URL encoding')
        return True

    # Check for path traversal in both original and decoded versions
    dangerous_patterns = ['..', '../', '..\\', '..\\\\']

    if any(pattern in user_input for pattern in dangerous_patterns):
        logger.error('Path traversal attack detected with invalid path')
        return True

    if any(pattern in decoded for pattern in dangerous_patterns):
        logger.error('Path traversal attack detected with invalid path')
        return True

    if any(pattern in double_decoded for pattern in dangerous_patterns):
        logger.error('Path traversal attack detected with invalid path')
        return True
    return False


def is_safe_path(safe_root, check_path, raw_file):
    """Detect Path Traversal."""
    if is_path_traversal(raw_file):
        return False
    safe_root = os.path.realpath(os.path.normpath(safe_root))
    check_path = os.path.realpath(os.path.normpath(check_path))
    return check_path.startswith(safe_root + os.sep) or check_path == safe_root


def clean_filename(filename, replace=' '):
    """Sanitize filename for Windows compatibility."""
    if system() == 'Windows':
        whitelist = f'-_.() {string.ascii_letters}{string.digits}'
        for r in replace:
            filename = filename.replace(r, '_')
        cleaned_filename = unicodedata.normalize(
            'NFKD', filename).encode('ASCII', 'ignore').decode()
        return ''.join(c for c in cleaned_filename if c in whitelist)
    return filename


def cmd_injection_check(data):
    """OS Cmd Injection check from Commix."""
    breakers = [
        ';', '%3B', '&', '%26', '&&',
        '%26%26', '|', '%7C', '||',
        '%7C%7C', '%0a', '%0d%0a',
    ]
    return any(i in data for i in breakers)


def is_pipe_or_link(path):
    """Check for named pipe or symlink."""
    return os.path.islink(path) or stat.S_ISFIFO(os.stat(path).st_mode)


def is_attack_pattern(user_input):
    """Check for shell injection attack patterns."""
    atk_pattern = re.compile(r';|\$\(|\|\||&&')
    result = re.findall(atk_pattern, user_input)
    if result:
        logger.error('Possible RCE attack detected')
    return result



# -*- coding: utf_8 -*-
import logging
from pathlib import Path

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    run_with_timeout,
    settings_enabled,
)

logger = logging.getLogger(__name__)


def apkid_analysis(checksum, apk_file):
    """APKID Analysis of DEX files."""
    if not settings_enabled('APKID_ENABLED'):
        return {}
    try:
        import apkid
    except ImportError as exp:
        msg = 'APKiD - Could not import APKiD'
        logger.error(msg)
        append_scan_status(checksum, msg, repr(exp))
        return {}

    apkid_ver = apkid.__version__
    from apkid.apkid import Scanner, Options
    from apkid.output import OutputFormatter
    from apkid.rules import RulesManager

    msg = f'Running APKiD {apkid_ver}'
    logger.info(msg)
    append_scan_status(checksum, msg)
    options = Options(
        timeout=30,
        verbose=False,
        entry_max_scan_size=settings.DATA_UPLOAD_MAX_MEMORY_SIZE,
        recursive=True,
    )
    output = OutputFormatter(
        json_output=True,
        output_dir=None,
        rules_manager=RulesManager(),
        include_types=False,
    )
    rules = options.rules_manager.load()
    scanner = Scanner(rules, options)
    findings = {}
    res = None
    try:
        res = run_with_timeout(
            scanner.scan_file,
            settings.BINARY_ANALYSIS_TIMEOUT,
            apk_file)
    except Exception as e:
        msg = 'APKID scan timed out'
        logger.error(msg)
        append_scan_status(
            checksum,
            msg,
            str(e))
    try:
        if res:
            findings = output._build_json_output(res)['files']
    except AttributeError:
        # apkid >= 2.0.3
        try:
            if res:
                findings = output.build_json_output(res)['files']
        except AttributeError:
            msg = (
                'yara-python dependency required by '
                'APKiD is not installed properly. '
                'Skipping APKiD analysis!')
            logger.error(msg)
            append_scan_status(
                checksum,
                msg,
                'Missing dependency')
    sanitized = {}
    for item in findings:
        filename = item['filename']
        if '!' in filename:
            filename = filename.split('!', 1)[1]
        else:
            filename = Path(filename).name
        sanitized[filename] = item['matches']
    return sanitized



# -*- coding: utf_8 -*-
"""Handle Classdump for iOS binaries."""

import logging
import os
import platform
import stat
import subprocess

from django.conf import settings

from mobsf.MobSF.utils import (
    append_scan_status,
    is_file_exists,
)


logger = logging.getLogger(__name__)


def classdump_mac(checksum, clsdmp_bin, tools_dir, ipa_bin):
    """Run Classdump for Objective-C/Swift."""
    if clsdmp_bin == 'class-dump-swift':
        external = settings.CLASSDUMP_SWIFT_BINARY
    else:
        external = settings.CLASSDUMP_BINARY
    msg = f'Running {clsdmp_bin} against binary'
    logger.info(msg)
    append_scan_status(checksum, msg)
    if (len(external) > 0
            and is_file_exists(external)):
        class_dump_bin = external
    else:
        class_dump_bin = os.path.join(
            tools_dir, clsdmp_bin)
    # Execute permission check
    if not os.access(class_dump_bin, os.X_OK):
        os.chmod(class_dump_bin, stat.S_IEXEC)
    return subprocess.check_output(
        [class_dump_bin, ipa_bin],
        stderr=subprocess.DEVNULL)


def classdump_linux(checksum, tools_dir, ipa_bin):
    """Run Classdump on Linux."""
    try:
        if (len(settings.JTOOL_BINARY) > 0
                and is_file_exists(settings.JTOOL_BINARY)):
            jtool_bin = settings.JTOOL_BINARY
        else:
            jtool_bin = os.path.join(tools_dir, 'jtool.ELF64')
        if not os.access(jtool_bin, os.X_OK):
            os.chmod(jtool_bin, stat.S_IEXEC)
        msg = 'Running jtool against the binary for dumping classes'
        logger.info(msg)
        append_scan_status(checksum, msg)
        args = [jtool_bin, '-arch', 'arm', '-d', 'objc', '-v', ipa_bin]
        # timeout to handle possible deadlock from jtool1
        return subprocess.check_output(
            args,
            stderr=subprocess.DEVNULL,
            timeout=60)
    except Exception:
        return b''


def get_class_dump(checksum, tools_dir, bin_path, app_dir, bin_type):
    """Running Classdump on binary."""
    try:
        bin_path = bin_path.as_posix()
        cdump = b''
        msg = 'Dumping Classes from the binary'
        logger.info(msg)
        append_scan_status(checksum, msg)
        if platform.system() == 'Darwin':
            if bin_type == 'Swift':
                try:
                    cdump = classdump_mac(
                        checksum,
                        'class-dump-swift',
                        tools_dir,
                        bin_path,
                    )
                except Exception:
                    cdump = classdump_mac(
                        checksum,
                        'class-dump',
                        tools_dir,
                        bin_path,
                    )
            else:
                try:
                    cdump = classdump_mac(
                        checksum,
                        'class-dump',
                        tools_dir,
                        bin_path,
                    )
                except Exception:
                    cdump = classdump_mac(
                        checksum,
                        'class-dump-swift',
                        tools_dir,
                        bin_path,
                    )
            if b'Source: (null)' in cdump:
                # Run failsafe if classdump failed
                msg = 'Running fail safe class-dump-swift'
                logger.info(msg)
                append_scan_status(checksum, msg)
                cdump = classdump_mac(
                    checksum,
                    'class-dump-swift',
                    tools_dir,
                    bin_path,
                )
        elif platform.system() == 'Linux':
            cdump = classdump_linux(checksum, tools_dir, bin_path)
        else:
            # Platform not supported
            msg = 'class-dump is not supported in this platform'
            logger.warning(msg)
            append_scan_status(checksum, msg)
        with open(os.path.join(app_dir, 'classdump.txt'), 'wb') as flip:
            flip.write(cdump)
        return cdump
    except Exception as exp:
        msg = 'Failed to dump classes from the binary'
        logger.exception(msg)
        append_scan_status(checksum, msg, repr(exp))
    return cdump



# -*- coding: utf_8 -*-
"""iOS Dynamic Analysis."""
import logging
import os
from pathlib import Path

from django.conf import settings
from django.shortcuts import render

from mobsf.MobSF.utils import (
    common_check,
    get_md5,
    print_n_send_error_response,
    strict_package_check,
)
from mobsf.DynamicAnalyzer.forms import UploadFileForm
from mobsf.DynamicAnalyzer.views.ios.helpers import (
    configure_proxy,
    get_local_ipa_list,
)
from mobsf.DynamicAnalyzer.views.ios.corellium_ssh import (
    generate_keypair_if_not_exists,
)
from mobsf.DynamicAnalyzer.views.ios.corellium_apis import (
    CorelliumAPI,
    CorelliumInstanceAPI,
)
from mobsf.MobSF.views.authentication import (
    login_required,
)
from mobsf.MobSF.views.authorization import (
    Permissions,
    permission_required,
)

logger = logging.getLogger(__name__)


@login_required
@permission_required(Permissions.SCAN)
def dynamic_analysis(request, api=False):
    """The iOS Dynamic Analysis Entry point."""
    try:
        scan_apps = get_local_ipa_list()
        # Corellium
        instances = []
        project_id = None
        c = CorelliumAPI(getattr(settings, 'CORELLIUM_PROJECT_ID', ''))
        corellium_auth = c.api_ready() and c.api_auth()
        if corellium_auth and c.get_projects():
            instances = c.get_instances()
            project_id = c.project_id
            setup_ssh_keys(c)
        context = {'apps': scan_apps,
                   'dynamic_analyzer': corellium_auth,
                   'project_id': project_id,
                   'instances': instances,
                   'title': 'MobSF Dynamic Analysis',
                   'version': settings.MOBSF_VER}
        if api:
            return context
        template = 'dynamic_analysis/ios/dynamic_analysis.html'
        return render(request, template, context)
    except Exception as exp:
        logger.exception('iOS Dynamic Analysis')
        return print_n_send_error_response(request, exp, api)


@login_required
@permission_required(Permissions.SCAN)
def dynamic_analyzer(request, api=False):
    """Dynamic Analyzer for in-device iOS apps."""
    try:
        if api:
            bundleid = request.POST.get('bundle_id')
            instance_id = request.POST.get('instance_id')
            form = None
        else:
            bundleid = request.GET.get('bundle_id')
            instance_id = request.GET.get('instance_id')
            form = UploadFileForm()
        if not bundleid or not strict_package_check(bundleid):
            return print_n_send_error_response(
                request,
                'Invalid iOS Bundle id',
                api)
        failed = common_check(instance_id)
        if failed:
            return print_n_send_error_response(
                request,
                failed['message'],
                api)
        bundle_hash = get_md5(bundleid.encode('utf-8'))
        app_dir = Path(settings.UPLD_DIR) / bundle_hash
        if not app_dir.exists():
            app_dir.mkdir()
        ci = CorelliumInstanceAPI(instance_id)
        configure_proxy(request, bundleid, ci)
        context = {
            'checksum': bundle_hash,
            'instance_id': instance_id,
            'bundle_id': bundleid,
            'version': settings.MOBSF_VER,
            'form': form,
            'title': 'iOS Dynamic Analyzer'}
        template = 'dynamic_analysis/ios/dynamic_analyzer.html'
        if api:
            return context
        return render(request, template, context)
    except Exception:
        logger.exception('iOS Dynamic Analyzer')
        return print_n_send_error_response(
            request,
            'iOS Dynamic Analysis Failed.',
            api)


def setup_ssh_keys(c):
    # Get Authorized keys for the project
    pkeys = c.get_authorized_keys()
    location = Path(settings.UPLD_DIR).parent
    _prv, pub = generate_keypair_if_not_exists(location)
    add_keys = False
    is_docker = os.getenv('MOBSF_PLATFORM') == 'docker'
    if not pkeys:
        # No SSH Keys associated with the project
        # let's add one
        add_keys = True
    else:
        # SSH Keys are already associated with the project
        # Check if our key is associated
        pub_key_exists = False
        for pkey in pkeys:
            if pkey['project'] == c.project_id:
                rkey = get_md5(pkey['key'].encode('utf-8'))
                if rkey == get_md5(pub):
                    pub_key_exists = True
                    break
            if is_docker and pkey['label'].endswith('(docker)'):
                # Delete all docker generated keys
                # This is done to avoid multiple stale keys being
                # added on each run.
                logger.info('Removing old stale SSH public key')
                c.delete_authorized_key(pkey['identifier'])
        # Our key is not asscoiated with the project, let's add it
        if not pub_key_exists:
            add_keys = True
    if add_keys:
        iden = c.add_authorized_key(pub)
        if not iden:
            logger.error('Failed to add SSH Key to Corellium project')
            return
        logger.info('Added SSH Key to Corellium project')
